"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Draft = void 0;
const clock_1 = require("./clock");
const PatchBuilder_1 = require("./PatchBuilder");
const Patch_1 = require("./Patch");
const DeleteOperation_1 = require("./operations/DeleteOperation");
const InsertArrayElementsOperation_1 = require("./operations/InsertArrayElementsOperation");
const InsertStringSubstringOperation_1 = require("./operations/InsertStringSubstringOperation");
const MakeArrayOperation_1 = require("./operations/MakeArrayOperation");
const MakeNumberOperation_1 = require("./operations/MakeNumberOperation");
const MakeObjectOperation_1 = require("./operations/MakeObjectOperation");
const MakeStringOperation_1 = require("./operations/MakeStringOperation");
const SetNumberOperation_1 = require("./operations/SetNumberOperation");
const SetObjectKeysOperation_1 = require("./operations/SetObjectKeysOperation");
const SetRootOperation_1 = require("./operations/SetRootOperation");
const NoopOperation_1 = require("./operations/NoopOperation");
const MakeConstantOperation_1 = require("./operations/MakeConstantOperation");
const MakeValueOperation_1 = require("./operations/MakeValueOperation");
class Draft {
    constructor() {
        this.builder = new PatchBuilder_1.PatchBuilder(new clock_1.LogicalVectorClock(-1, 0));
    }
    patch(clock) {
        const patch = new Patch_1.Patch();
        const ops = this.builder.patch.ops;
        const ts = (id) => id.getSessionId() !== -1 ? id : new clock_1.LogicalTimestamp(clock.getSessionId(), clock.time + id.time);
        for (const op of ops) {
            if (op instanceof DeleteOperation_1.DeleteOperation)
                patch.ops.push(new DeleteOperation_1.DeleteOperation(ts(op.id), ts(op.obj), ts(op.after), op.length));
            else if (op instanceof InsertArrayElementsOperation_1.InsertArrayElementsOperation)
                patch.ops.push(new InsertArrayElementsOperation_1.InsertArrayElementsOperation(ts(op.id), ts(op.arr), ts(op.after), op.elements.map(ts)));
            else if (op instanceof InsertStringSubstringOperation_1.InsertStringSubstringOperation)
                patch.ops.push(new InsertStringSubstringOperation_1.InsertStringSubstringOperation(ts(op.id), ts(op.obj), ts(op.after), op.substring));
            else if (op instanceof MakeArrayOperation_1.MakeArrayOperation)
                patch.ops.push(new MakeArrayOperation_1.MakeArrayOperation(ts(op.id)));
            else if (op instanceof MakeConstantOperation_1.MakeConstantOperation)
                patch.ops.push(new MakeConstantOperation_1.MakeConstantOperation(ts(op.id), op.value));
            else if (op instanceof MakeValueOperation_1.MakeValueOperation)
                patch.ops.push(new MakeValueOperation_1.MakeValueOperation(ts(op.id), op.value));
            else if (op instanceof MakeNumberOperation_1.MakeNumberOperation)
                patch.ops.push(new MakeNumberOperation_1.MakeNumberOperation(ts(op.id)));
            else if (op instanceof MakeObjectOperation_1.MakeObjectOperation)
                patch.ops.push(new MakeObjectOperation_1.MakeObjectOperation(ts(op.id)));
            else if (op instanceof MakeStringOperation_1.MakeStringOperation)
                patch.ops.push(new MakeStringOperation_1.MakeStringOperation(ts(op.id)));
            else if (op instanceof SetNumberOperation_1.SetNumberOperation)
                patch.ops.push(new SetNumberOperation_1.SetNumberOperation(ts(op.id), ts(op.num), op.value));
            else if (op instanceof SetObjectKeysOperation_1.SetObjectKeysOperation)
                patch.ops.push(new SetObjectKeysOperation_1.SetObjectKeysOperation(ts(op.id), ts(op.object), op.tuples.map(([key, value]) => [key, ts(value)])));
            else if (op instanceof SetRootOperation_1.SetRootOperation)
                patch.ops.push(new SetRootOperation_1.SetRootOperation(ts(op.id), ts(op.value)));
            else if (op instanceof NoopOperation_1.NoopOperation)
                patch.ops.push(new NoopOperation_1.NoopOperation(ts(op.id), op.length));
        }
        return patch;
    }
}
exports.Draft = Draft;
