"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServerVectorClock = exports.ServerClock = exports.ServerTimespan = exports.ServerTimestamp = void 0;
const SERVER_SESSION_ID = 1;
class ServerTimestamp {
    constructor(time) {
        this.time = time;
    }
    getSessionId() {
        return SERVER_SESSION_ID;
    }
    isEqual(ts) {
        return this.getSessionId() === ts.getSessionId() && this.time === ts.time;
    }
    compare(ts) {
        if (this.time > ts.time)
            return 1;
        if (this.time < ts.time)
            return -1;
        if (this.getSessionId() > ts.getSessionId())
            return 1;
        if (this.getSessionId() < ts.getSessionId())
            return -1;
        return 0;
    }
    inSpan(span, ts, tsSpan) {
        if (this.getSessionId() !== ts.getSessionId())
            return false;
        if (this.time > ts.time)
            return false;
        if (this.time + span < ts.time + tsSpan)
            return false;
        return true;
    }
    overlap(span, ts, tsSpan) {
        if (this.getSessionId() !== ts.getSessionId())
            return 0;
        const x1 = this.time;
        const x2 = x1 + span;
        const y1 = ts.time;
        const y2 = y1 + tsSpan;
        const min = Math.max(x1, y1);
        const max = Math.min(x2, y2);
        const diff = max - min;
        return diff <= 0 ? 0 : diff;
    }
    tick(cycles) {
        return new ServerTimestamp(this.time + cycles);
    }
    interval(cycles, span) {
        return new ServerTimespan(this.time + cycles, span);
    }
    toString() {
        return this.getSessionId() + '!' + this.time;
    }
    toDisplayString() {
        let session = String(this.getSessionId());
        if (session.length > 4)
            session = '..' + session.substr(session.length - 4);
        return session + '!' + this.time;
    }
    clock() {
        return new ServerClock(this.time);
    }
    compact() {
        return this.getSessionId() + ',' + this.time;
    }
}
exports.ServerTimestamp = ServerTimestamp;
class ServerTimespan extends ServerTimestamp {
    constructor(time, span) {
        super(time);
        this.span = span;
    }
}
exports.ServerTimespan = ServerTimespan;
class ServerClock extends ServerTimestamp {
    tick(cycles) {
        const timestamp = new ServerTimestamp(this.time);
        this.time += cycles;
        return timestamp;
    }
}
exports.ServerClock = ServerClock;
class ServerVectorClock extends ServerClock {
    constructor() {
        super(...arguments);
        this.clocks = new Map();
    }
    observe(ts, span) {
        if (ts.getSessionId() !== SERVER_SESSION_ID)
            throw new Error('INVALID_SERVER_SESSION');
        if (this.time < ts.time)
            throw new Error('TIME_TRAVEL');
        const time = ts.time + span;
        if (time > this.time)
            this.time = time;
    }
    clone() {
        return this.fork();
    }
    fork() {
        return new ServerVectorClock(this.time);
    }
}
exports.ServerVectorClock = ServerVectorClock;
