"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decode = void 0;
const clock_1 = require("../../clock");
const PatchBuilder_1 = require("../../PatchBuilder");
const decode = (data) => {
    const sessionId = data[0];
    const time = data[1];
    const clock = new clock_1.LogicalVectorClock(sessionId, time);
    const builder = new PatchBuilder_1.PatchBuilder(clock);
    const length = data.length;
    let i = 2;
    const decodeTimestamp = () => {
        const x = data[i++];
        if (x < 0)
            return new clock_1.LogicalTimestamp(sessionId, time - x - 1);
        else
            return new clock_1.LogicalTimestamp(x, data[i++]);
    };
    while (i < length) {
        switch (data[i++]) {
            case 0: {
                builder.obj();
                break;
            }
            case 1: {
                builder.arr();
                break;
            }
            case 2: {
                builder.str();
                break;
            }
            case 3: {
                builder.num();
                break;
            }
            case 4: {
                builder.root(decodeTimestamp());
                break;
            }
            case 5: {
                const length = data[i++];
                const obj = decodeTimestamp();
                const tuples = [];
                for (let j = 0; j < length; j++) {
                    const key = data[i++];
                    tuples.push([key, decodeTimestamp()]);
                }
                builder.setKeys(obj, tuples);
                break;
            }
            case 6: {
                const value = data[i++];
                builder.setNum(decodeTimestamp(), value);
                break;
            }
            case 7: {
                const value = data[i++];
                builder.insStr(decodeTimestamp(), decodeTimestamp(), value);
                break;
            }
            case 8: {
                const length = data[i++];
                const arr = decodeTimestamp();
                const after = decodeTimestamp();
                const values = [];
                for (let j = 0; j < length; j++)
                    values.push(decodeTimestamp());
                builder.insArr(arr, after, values);
                break;
            }
            case 9: {
                const obj = decodeTimestamp();
                const after = decodeTimestamp();
                builder.del(obj, after, 1);
                break;
            }
            case 10: {
                const span = data[i++];
                const obj = decodeTimestamp();
                const after = decodeTimestamp();
                builder.del(obj, after, span);
                break;
            }
            case 11: {
                builder.noop(1);
                break;
            }
            case 12: {
                builder.noop(data[i++]);
                break;
            }
            case 13: {
                builder.const(data[i++]);
                break;
            }
            case 14: {
                builder.val(data[i++]);
                break;
            }
            case 15: {
                const obj = decodeTimestamp();
                const value = data[i++];
                builder.setVal(obj, value);
                break;
            }
        }
    }
    return builder.patch;
};
exports.decode = decode;
