"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.encode = void 0;
const DeleteOperation_1 = require("../../operations/DeleteOperation");
const InsertArrayElementsOperation_1 = require("../../operations/InsertArrayElementsOperation");
const InsertStringSubstringOperation_1 = require("../../operations/InsertStringSubstringOperation");
const MakeArrayOperation_1 = require("../../operations/MakeArrayOperation");
const MakeConstantOperation_1 = require("../../operations/MakeConstantOperation");
const MakeNumberOperation_1 = require("../../operations/MakeNumberOperation");
const MakeObjectOperation_1 = require("../../operations/MakeObjectOperation");
const MakeStringOperation_1 = require("../../operations/MakeStringOperation");
const MakeValueOperation_1 = require("../../operations/MakeValueOperation");
const NoopOperation_1 = require("../../operations/NoopOperation");
const SetNumberOperation_1 = require("../../operations/SetNumberOperation");
const SetObjectKeysOperation_1 = require("../../operations/SetObjectKeysOperation");
const SetRootOperation_1 = require("../../operations/SetRootOperation");
const SetValueOperation_1 = require("../../operations/SetValueOperation");
const encodeTimestamp = (ts) => [ts.getSessionId(), ts.time];
const encode = (patch) => {
    const id = patch.getId();
    if (!id)
        throw new Error('PATCH_EMPTY');
    const ops = [];
    const res = {
        id: encodeTimestamp(id),
        ops,
    };
    for (const op of patch.ops) {
        if (op instanceof MakeObjectOperation_1.MakeObjectOperation) {
            ops.push({ op: 'obj' });
            continue;
        }
        if (op instanceof MakeArrayOperation_1.MakeArrayOperation) {
            ops.push({ op: 'arr' });
            continue;
        }
        if (op instanceof MakeStringOperation_1.MakeStringOperation) {
            ops.push({ op: 'str' });
            continue;
        }
        if (op instanceof MakeNumberOperation_1.MakeNumberOperation) {
            ops.push({ op: 'num' });
            continue;
        }
        if (op instanceof MakeValueOperation_1.MakeValueOperation) {
            ops.push({ op: 'val', value: op.value });
            continue;
        }
        if (op instanceof MakeConstantOperation_1.MakeConstantOperation) {
            ops.push({ op: 'const', value: op.value });
            continue;
        }
        if (op instanceof SetRootOperation_1.SetRootOperation) {
            ops.push({
                op: 'root',
                value: encodeTimestamp(op.value),
            });
            continue;
        }
        if (op instanceof SetObjectKeysOperation_1.SetObjectKeysOperation) {
            ops.push({
                op: 'obj_set',
                obj: encodeTimestamp(op.object),
                tuples: op.tuples.map(([key, value]) => [key, encodeTimestamp(value)]),
            });
            continue;
        }
        if (op instanceof SetNumberOperation_1.SetNumberOperation) {
            ops.push({
                op: 'num_set',
                after: encodeTimestamp(op.num),
                value: op.value,
            });
            continue;
        }
        if (op instanceof SetValueOperation_1.SetValueOperation) {
            ops.push({
                op: 'val_set',
                obj: encodeTimestamp(op.obj),
                value: op.value,
            });
            continue;
        }
        if (op instanceof InsertStringSubstringOperation_1.InsertStringSubstringOperation) {
            ops.push({
                op: 'str_ins',
                obj: encodeTimestamp(op.obj),
                after: encodeTimestamp(op.after),
                value: op.substring,
            });
            continue;
        }
        if (op instanceof InsertArrayElementsOperation_1.InsertArrayElementsOperation) {
            ops.push({
                op: 'arr_ins',
                obj: encodeTimestamp(op.arr),
                after: encodeTimestamp(op.after),
                values: op.elements.map(encodeTimestamp),
            });
            continue;
        }
        if (op instanceof DeleteOperation_1.DeleteOperation) {
            const encoded = {
                op: 'del',
                obj: encodeTimestamp(op.obj),
                after: encodeTimestamp(op.after),
            };
            const span = op.span();
            if (span > 1)
                encoded.len = span;
            ops.push(encoded);
            continue;
        }
        if (op instanceof NoopOperation_1.NoopOperation) {
            const encoded = {
                op: 'noop',
            };
            const length = op.length;
            if (length > 1)
                encoded.len = length;
            ops.push(encoded);
            continue;
        }
    }
    return res;
};
exports.encode = encode;
