"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpCopy = void 0;
const AbstractOp_1 = require("./AbstractOp");
const json_pointer_1 = require("../../json-pointer");
const OpAdd_1 = require("./OpAdd");
const util_1 = require("../util");
class OpCopy extends AbstractOp_1.AbstractOp {
    constructor(path, from) {
        super(path);
        this.from = from;
    }
    op() {
        return 'copy';
    }
    apply(doc) {
        const { val } = json_pointer_1.find(doc, this.from);
        if (val === undefined)
            throw new Error('NOT_FOUND');
        const add = new OpAdd_1.OpAdd(this.path, util_1.deepClone(val)).apply(doc);
        return add;
    }
    toJson(parent) {
        return {
            op: 'copy',
            path: json_pointer_1.formatJsonPointer(this.path),
            from: json_pointer_1.formatJsonPointer(this.from),
        };
    }
    toCompact(parent) {
        return [3, this.path, this.from];
    }
    encode(encoder, parent) {
        encoder.encodeArrayHeader(3);
        encoder.u8(3);
        encoder.encodeArray(this.path);
        encoder.encodeArray(this.from);
    }
}
exports.OpCopy = OpCopy;
