"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpIn = void 0;
const json_pointer_1 = require("../../json-pointer");
const AbstractPredicateOp_1 = require("./AbstractPredicateOp");
const isEqual = require('fast-deep-equal');
class OpIn extends AbstractPredicateOp_1.AbstractPredicateOp {
    constructor(path, value) {
        super(path);
        this.value = value;
    }
    op() {
        return 'in';
    }
    test(doc) {
        const { val } = json_pointer_1.find(doc, this.path);
        for (const x of this.value)
            if (isEqual(val, x))
                return true;
        return false;
    }
    toJson(parent) {
        const op = {
            op: 'in',
            path: json_pointer_1.formatJsonPointer(parent ? this.path.slice(parent.path.length) : this.path),
            value: this.value,
        };
        return op;
    }
    toCompact(parent) {
        return [33, parent ? this.path.slice(parent.path.length) : this.path, this.value];
    }
    encode(encoder, parent) {
        encoder.encodeArrayHeader(3);
        encoder.u8(33);
        encoder.encodeArray(parent ? this.path.slice(parent.path.length) : this.path);
        encoder.encodeArray(this.value);
    }
}
exports.OpIn = OpIn;
