"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PersistentClient = void 0;
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const rpc_1 = require("../rpc");
const RpcApiCaller_1 = require("../rpc/RpcApiCaller");
const RpcDuplex_1 = require("../rpc/RpcDuplex");
const RpcServer_1 = require("../rpc/RpcServer");
const channel_1 = require("./channel");
class PersistentClient {
    constructor(params) {
        var _a;
        this.rpc$ = new rxjs_1.ReplaySubject(1);
        const ping = (_a = params.ping) !== null && _a !== void 0 ? _a : 15000;
        this.channel = new channel_1.PersistentChannel(params.channel);
        this.channel.open$.pipe(operators_1.filter(open => open)).subscribe(() => {
            const close$ = this.channel.open$.pipe(operators_1.filter(open => !open));
            const duplex = new RpcDuplex_1.RpcDuplex({
                client: new rpc_1.RpcClient(Object.assign(Object.assign({}, (params.client || {})), { send: (messages) => {
                        const encoded = params.codec.encoder.encode(messages);
                        this.channel.send$(encoded).subscribe();
                    } })),
                server: new RpcServer_1.RpcServer(Object.assign(Object.assign({}, (params.server || {
                    caller: new RpcApiCaller_1.RpcApiCaller({
                        api: {},
                    }),
                    onNotification: () => { },
                })), { send: (messages) => {
                        const encoded = params.codec.encoder.encode(messages);
                        this.channel.send$(encoded).subscribe();
                    } })),
            });
            this.channel.message$
                .pipe(operators_1.takeUntil(close$))
                .subscribe(data => {
                const encoded = typeof data === 'string' ? data : new Uint8Array(data);
                const messages = params.codec.decoder.decode(encoded);
                duplex.onMessages((messages instanceof Array ? messages : [messages]), {});
            });
            if (ping) {
                rxjs_1.timer(ping, ping)
                    .pipe(operators_1.takeUntil(close$))
                    .subscribe(() => {
                    duplex.notify('.ping', undefined);
                });
            }
            if (this.rpc)
                this.rpc.disconnect();
            this.rpc = duplex;
            this.rpc$.next(duplex);
        });
    }
    call$(method, data) {
        return this.rpc$
            .pipe(operators_1.first(), operators_1.switchMap(rpc => rpc.call$(method, data)));
    }
    call(method, data) {
        return rxjs_1.firstValueFrom(this.call$(method, data));
    }
    notify(method, data) {
        this.rpc$
            .subscribe(rpc => rpc.notify(method, data));
    }
    start() {
        this.channel.start();
    }
    stop() {
        this.channel.stop();
        if (this.rpc)
            this.rpc.stop();
    }
}
exports.PersistentClient = PersistentClient;
