"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decodeFullMessages = exports.decodeFullMessage = void 0;
const binary_1 = require("../../messages/binary");
const header_1 = require("./header");
const readMethod = (arr, offset) => {
    const size = arr[offset++];
    let str = '';
    for (let i = 0; i < size; i++)
        str += String.fromCharCode(arr[offset++]);
    return [str, offset];
};
const decodeFullMessage = (arr, offset) => {
    const byte1 = arr[offset++];
    const code = byte1 >>> 5;
    switch (code) {
        case 3: {
            const [length, off1] = header_1.readHeader(byte1, arr, offset);
            const [method, off2] = readMethod(arr, off1);
            const data = arr.subarray(off2, off2 + length);
            return [new binary_1.BinaryNotificationMessage(method, data), off2 + length];
        }
        case 4:
        case 5:
        case 6: {
            const [length, off1] = header_1.readHeader(byte1, arr, offset);
            const o1 = arr[off1];
            const o2 = arr[off1 + 1];
            const id = (o1 << 8) | o2;
            offset = off1 + 2;
            const data = arr.subarray(offset, offset + length);
            const message = code === 4
                ? new binary_1.BinaryResponseDataMessage(id, data)
                : code === 5
                    ? new binary_1.BinaryResponseCompleteMessage(id, data)
                    : new binary_1.BinaryResponseErrorMessage(id, data);
            return [message, offset + length];
        }
        case 0:
        case 1:
        case 2: {
            const [length, off1] = header_1.readHeader(byte1, arr, offset);
            const o1 = arr[off1];
            const o2 = arr[off1 + 1];
            const id = (o1 << 8) | o2;
            const [method, off2] = readMethod(arr, off1 + 2);
            const data = arr.subarray(off2, off2 + length);
            const message = code === 0
                ? new binary_1.BinaryRequestDataMessage(id, method, data)
                : code === 1
                    ? new binary_1.BinaryRequestCompleteMessage(id, method, data)
                    : new binary_1.BinaryRequestErrorMessage(id, method, data);
            return [message, off2 + length];
        }
        case 0b111: {
            switch (byte1) {
                case 255: {
                    const o1 = arr[offset++];
                    const o2 = arr[offset++];
                    const id = (o1 << 8) | o2;
                    return [new binary_1.BinaryResponseUnsubscribeMessage(id), offset];
                }
                case 254: {
                    const o1 = arr[offset++];
                    const o2 = arr[offset++];
                    const id = (o1 << 8) | o2;
                    return [new binary_1.BinaryRequestUnsubscribeMessage(id), offset];
                }
            }
        }
    }
    throw new Error('UNKNOWN_MESSAGE');
};
exports.decodeFullMessage = decodeFullMessage;
const decodeFullMessages = (arr, offset, end) => {
    const messages = [];
    while (offset < end) {
        const [message, off] = exports.decodeFullMessage(arr, offset);
        offset = off;
        messages.push(message);
    }
    return messages;
};
exports.decodeFullMessages = decodeFullMessages;
