"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RpcDuplex = void 0;
const nominal_1 = require("../messages/nominal");
class RpcDuplex {
    constructor(params) {
        this.client = params.client;
        this.server = params.server;
    }
    onMessages(messages, ctx) {
        const length = messages.length;
        for (let i = 0; i < length; i++)
            this.onMessage(messages[i], ctx);
    }
    onMessage(message, ctx) {
        if (message instanceof nominal_1.RequestDataMessage)
            this.server.onRequestDataMessage(message, ctx);
        else if (message instanceof nominal_1.RequestCompleteMessage)
            this.server.onRequestCompleteMessage(message, ctx);
        else if (message instanceof nominal_1.RequestErrorMessage)
            this.server.onRequestErrorMessage(message, ctx);
        else if (message instanceof nominal_1.ResponseUnsubscribeMessage)
            this.server.onUnsubscribeMessage(message);
        else if (message instanceof nominal_1.NotificationMessage)
            this.server.onNotificationMessage(message, ctx);
        else if (message instanceof nominal_1.ResponseCompleteMessage)
            return this.client.onResponseComplete(message);
        else if (message instanceof nominal_1.ResponseDataMessage)
            return this.client.onResponseData(message);
        else if (message instanceof nominal_1.ResponseErrorMessage)
            return this.client.onResponseError(message);
        else if (message instanceof nominal_1.RequestUnsubscribeMessage)
            return this.client.onRequestUnsubscribe(message);
    }
    call$(method, data) {
        return this.client.call$(method, data);
    }
    call(method, data) {
        return this.client.call(method, data);
    }
    notify(method, data) {
        this.client.notify(method, data);
    }
    stop() {
        this.client.stop();
        this.server.stop();
    }
    disconnect() {
        this.client.disconnect();
        this.server.disconnect();
    }
}
exports.RpcDuplex = RpcDuplex;
