"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RpcPersistentChannel = void 0;
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const rpc_1 = require("../rpc");
const RpcDuplex_1 = require("../rpc/RpcDuplex");
const RpcServer_1 = require("../rpc/RpcServer");
const channel_1 = require("../channel/channel");
class RpcPersistentChannel {
    constructor(params) {
        this.rpc$ = new rxjs_1.ReplaySubject(1);
        this.channel = new channel_1.PersistentChannel(params.channel);
        this.channel.open$.pipe(operators_1.filter(open => open)).subscribe(() => {
            const close$ = this.channel.open$.pipe(operators_1.filter(open => !open));
            const rpc = new RpcDuplex_1.RpcDuplex({
                client: new rpc_1.RpcClient(Object.assign(Object.assign({}, params.client), { send: (messages) => {
                        const encoded = params.codec.encoder.encode(messages);
                        this.channel.send$(encoded).subscribe();
                    } })),
                server: new RpcServer_1.RpcServer(Object.assign(Object.assign({}, params.server), { send: (messages) => {
                        const encoded = params.codec.encoder.encode(messages);
                        this.channel.send$(encoded).subscribe();
                    } })),
            });
            this.channel.message$
                .pipe(operators_1.takeUntil(close$))
                .subscribe(data => {
                const encoded = typeof data === 'string' ? data : new Uint8Array(data);
                const messages = params.codec.decoder.decode(encoded);
                rpc.onMessages(messages, {});
            });
            if (this.rpc)
                this.rpc.disconnect();
            this.rpc = rpc;
            this.rpc$.next(rpc);
        });
    }
    start() {
        this.channel.start();
    }
    stop() {
        var _a;
        (_a = this.rpc) === null || _a === void 0 ? void 0 : _a.stop();
        this.channel.stop();
    }
    call$(method, data) {
        return this.rpc$
            .pipe(operators_1.switchMap(rpc => rpc.call$(method, data)));
    }
    notify(method, data) {
        this.rpc$
            .subscribe(rpc => rpc.notify(method, data));
    }
}
exports.RpcPersistentChannel = RpcPersistentChannel;
