"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RpcError = exports.ErrorLikeErrorFormatter = exports.formatErrorCode = exports.formatError = exports.isErrorLike = exports.formatErrorLike = void 0;
const constants_1 = require("./constants");
const formatErrorLike = (error) => {
    const out = { message: error.message };
    if (typeof error.status === 'number')
        out.status = error.status;
    if (typeof error.code === 'string')
        out.code = error.code;
    if (typeof error.errno === 'number')
        out.errno = error.errno;
    if (typeof error.errorId === 'number')
        out.errorId = error.errorId;
    return out;
};
exports.formatErrorLike = formatErrorLike;
const isErrorLike = (error) => {
    if (error instanceof Error)
        return true;
    if (typeof error === 'object')
        if (typeof error.message === 'string')
            return true;
    return false;
};
exports.isErrorLike = isErrorLike;
const formatError = (error) => {
    if (exports.isErrorLike(error))
        return exports.formatErrorLike(error);
    return {
        message: String(error),
    };
};
exports.formatError = formatError;
const formatErrorCode = (errno) => {
    return {
        message: 'PROTOCOL',
        errno,
        code: constants_1.RpcServerError[errno],
    };
};
exports.formatErrorCode = formatErrorCode;
class ErrorLikeErrorFormatter {
    format(error) {
        return exports.formatError(error);
    }
    formatValidation(error) {
        return this.format(error);
    }
    formatCode(code) {
        return exports.formatErrorCode(code);
    }
}
exports.ErrorLikeErrorFormatter = ErrorLikeErrorFormatter;
class RpcError extends Error {
    constructor(errno, message = 'PROTOCOL') {
        super(message);
        this.errno = errno;
        this.code = constants_1.RpcServerError[errno];
    }
}
exports.RpcError = RpcError;
