import type { Observable } from "rxjs";
export declare type RpcMethod<Context = unknown, Request = unknown, Response = unknown> = RpcMethodStatic<Context, Request, Response> | RpcMethodStreaming<Context, Request, Response>;
export declare type RpcMethodRequest<T> = T extends RpcMethod<any, infer U, any> ? U : never;
export declare type RpcMethodResponse<T> = T extends RpcMethod<any, any, infer U> ? U : never;
export interface RpcMethodBase<Context = unknown, Request = unknown> {
    isStreaming: boolean;
    validate?(request: Request): void;
    onPreCall?(ctx: Context, request: Request): Promise<void>;
    pretty?: boolean;
}
export interface RpcMethodStatic<Context = unknown, Request = unknown, Response = unknown> extends RpcMethodBase<Context, Request> {
    isStreaming: false;
    call(ctx: Context, request: Request): Promise<Response>;
}
export interface RpcMethodStreaming<Context = unknown, Request = unknown, Response = unknown> extends RpcMethodBase<Context, Request> {
    isStreaming: true;
    preCallBufferSize?: number;
    timeout?: number;
    call$(ctx: Context, request$: Observable<Request>): Observable<Response>;
}
export declare type RpcApi<Context = unknown, T = unknown> = Record<string, RpcMethod<Context, T, T>>;
export interface IRpcApiCaller<Api extends Record<string, RpcMethod<Ctx, any, any>>, Ctx = unknown> {
    exists<K extends keyof Api>(name: K): boolean;
    get<K extends keyof Api>(name: K): Api[K];
    call<K extends keyof Api>(name: K, request: RpcMethodRequest<Api[K]>, ctx: Ctx): Promise<RpcMethodResponse<Api[K]>>;
    call$<K extends keyof Api>(name: K, request$: Observable<RpcMethodRequest<Api[K]>>, ctx: Ctx): Observable<RpcMethodResponse<Api[K]>>;
}
