"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableWsBinaryReactiveRpcApi = void 0;
const binary_msgpack_1 = require("../../../common/codec/binary-msgpack");
const context_1 = require("../context");
const enableWsBinaryReactiveRpcApi = (params) => {
    const encoder = new binary_msgpack_1.Encoder();
    const decoder = new binary_msgpack_1.Decoder();
    const { route = '/rpc/binary', uws, createRpcServer, onNotification, createContext = context_1.createConnectionContext, compression, idleTimeout = 0, maxBackpressure = 3145728, maxPayloadLength = 1048576, } = params;
    uws.ws(route, {
        idleTimeout,
        compression,
        maxBackpressure,
        maxPayloadLength,
        upgrade: (res, req, context) => {
            const secWebSocketKey = req.getHeader('sec-websocket-key');
            const secWebSocketProtocol = req.getHeader('sec-websocket-protocol');
            const secWebSocketExtensions = req.getHeader('sec-websocket-extensions');
            const ctx = createContext(req, res);
            res.upgrade({ ctx }, secWebSocketKey, secWebSocketProtocol, secWebSocketExtensions, context);
        },
        open: (ws) => {
            const rpc = createRpcServer({
                send: (messages) => {
                    if (ws.getBufferedAmount() > maxBackpressure)
                        return;
                    const uint8 = encoder.encode(messages);
                    ws.send(uint8, true);
                },
            });
            if (onNotification) {
                rpc.onNotification = (name, data, ctx) => {
                    onNotification(ws, name, data, ctx);
                };
            }
            ws.rpc = rpc;
        },
        message: (ws, buf, isBinary) => {
            const { ctx, rpc } = ws;
            const uint8 = new Uint8Array(buf);
            const messages = decoder.decode(uint8);
            const length = messages.length;
            for (let i = 0; i < length; i++) {
                const message = messages[i];
                rpc.onMessage(message, ctx);
            }
        },
        close: (ws, code, message) => {
            ws.rpc.stop();
        },
    });
};
exports.enableWsBinaryReactiveRpcApi = enableWsBinaryReactiveRpcApi;
