export class LogicalTimestamp {
    constructor(sessionId, time) {
        this.sessionId = sessionId;
        this.time = time;
    }
    getSessionId() {
        return this.sessionId;
    }
    isEqual(ts) {
        return this.getSessionId() === ts.getSessionId() && this.time === ts.time;
    }
    compare(ts) {
        if (this.time > ts.time)
            return 1;
        if (this.time < ts.time)
            return -1;
        if (this.getSessionId() > ts.getSessionId())
            return 1;
        if (this.getSessionId() < ts.getSessionId())
            return -1;
        return 0;
    }
    inSpan(span, ts, tsSpan) {
        if (this.getSessionId() !== ts.getSessionId())
            return false;
        if (this.time > ts.time)
            return false;
        if (this.time + span < ts.time + tsSpan)
            return false;
        return true;
    }
    overlap(span, ts, tsSpan) {
        if (this.getSessionId() !== ts.getSessionId())
            return 0;
        const x1 = this.time;
        const x2 = x1 + span;
        const y1 = ts.time;
        const y2 = y1 + tsSpan;
        const min = Math.max(x1, y1);
        const max = Math.min(x2, y2);
        const diff = max - min;
        return diff <= 0 ? 0 : diff;
    }
    tick(cycles) {
        return new LogicalTimestamp(this.getSessionId(), this.time + cycles);
    }
    interval(cycles, span) {
        return new LogicalTimespan(this.getSessionId(), this.time + cycles, span);
    }
    toString() {
        return this.getSessionId() + '!' + this.time;
    }
    toDisplayString() {
        let session = String(this.getSessionId());
        if (session.length > 4)
            session = '..' + session.substr(session.length - 4);
        return session + '!' + this.time;
    }
    clock() {
        return new LogicalClock(this.getSessionId(), this.time);
    }
    compact() {
        return this.getSessionId() + ',' + this.time;
    }
}
export class LogicalTimespan extends LogicalTimestamp {
    constructor(sessionId, time, span) {
        super(sessionId, time);
        this.span = span;
    }
}
export class LogicalClock extends LogicalTimestamp {
    tick(cycles) {
        const timestamp = new LogicalTimestamp(this.sessionId, this.time);
        this.time += cycles;
        return timestamp;
    }
}
export class LogicalVectorClock extends LogicalClock {
    constructor() {
        super(...arguments);
        this.clocks = new Map();
    }
    observe(ts, span) {
        const time = ts.time + span - 1;
        const clock = this.clocks.get(ts.getSessionId());
        if (!clock)
            this.clocks.set(ts.getSessionId(), ts.tick(span - 1));
        else if (time > clock.time)
            clock.time = ts.time;
        if (time >= this.time)
            this.time = time + 1;
    }
    clone() {
        return this.fork(this.getSessionId());
    }
    fork(sessionId) {
        const clock = new LogicalVectorClock(sessionId, this.time);
        for (const ts of this.clocks.values())
            clock.observe(ts.tick(0), 1);
        return clock;
    }
}
