import { LogicalTimestamp, LogicalVectorClock } from '../../clock';
export class ClockDecoder {
    constructor(sessionId, time) {
        this.table = new Map();
        this.index = 1;
        this.clock = new LogicalVectorClock(sessionId, time);
        if (time)
            this.clock.observe(new LogicalTimestamp(sessionId, time), 1);
        this.table.set(this.index++, this.clock);
    }
    static fromArr(arr) {
        const decoder = new ClockDecoder(arr[0], arr[1]);
        const length = arr.length;
        for (let i = 2; i < length; i += 2)
            decoder.pushTuple(arr[i], arr[i + 1]);
        return decoder;
    }
    pushTuple(sessionId, time) {
        const ts = new LogicalTimestamp(sessionId, time);
        this.clock.observe(ts, 1);
        this.table.set(this.index++, ts);
    }
    decodeId(sessionIndex, timeDiff) {
        if (!sessionIndex)
            return new LogicalTimestamp(0, timeDiff);
        const ts = this.table.get(sessionIndex);
        if (!ts)
            throw new Error('INVALID_CLOCK_TABLE');
        return new LogicalTimestamp(ts.getSessionId(), ts.time - timeDiff);
    }
}
