import { RelativeTimestamp } from './RelativeTimestamp';
export class ClockEncoder {
    constructor(clock) {
        this.clock = clock;
        this.index = 1;
        this.table = new Map();
        this.table.set(clock.getSessionId(), this.index++);
    }
    append(ts) {
        const { time } = ts;
        const sessionId = ts.getSessionId();
        if (sessionId === 0)
            return new RelativeTimestamp(0, ts.time);
        const clock = this.clock.clocks.get(sessionId);
        if (!clock)
            throw new Error(`Clock not found (${sessionId}, ${time}).`);
        if (!this.table.has(sessionId))
            this.table.set(sessionId, this.index++);
        const sessionIndex = this.table.get(sessionId);
        const timeDiff = clock.time - time;
        const relativeTimestamp = new RelativeTimestamp(sessionIndex, timeDiff);
        return relativeTimestamp;
    }
    toJson() {
        const out = [];
        for (const sessionId of this.table.keys()) {
            const clock = this.clock.clocks.get(sessionId);
            if (clock)
                out.push(clock.getSessionId(), clock.time);
            else if (this.clock.getSessionId() === sessionId) {
                out.push(sessionId, this.clock.time);
            }
            else {
            }
        }
        return out;
    }
    *clocks() {
        for (const sessionId of this.table.keys())
            yield this.clock.clocks.get(sessionId);
    }
}
