import { ConstantType } from '../../types/const/ConstantType';
import { ObjectType } from '../../types/lww-object/ObjectType';
import { ValueType } from '../../types/lww-value/ValueType';
import { ArrayType } from '../../types/rga-array/ArrayType';
import { StringType } from '../../types/rga-string/StringType';
export class AbstractEncoder {
    encodeRoot(arr, root) {
        this.ts(arr, root.id);
        if (!root.node)
            arr.push(0);
        else
            this.encodeNode(arr, root.node);
    }
    encodeNode(arr, node) {
        if (node instanceof ObjectType)
            return this.encodeObj(arr, node);
        else if (node instanceof ArrayType)
            return this.encodeArr(arr, node);
        else if (node instanceof StringType)
            return this.encodeStr(arr, node);
        else if (node instanceof ValueType)
            return this.encodeVal(arr, node);
        else if (node instanceof ConstantType)
            return this.encodeConst(arr, node);
        throw new Error('UNKNOWN_NODE');
    }
    encodeObj(arr, obj) {
        const res = [0];
        arr.push(res);
        this.ts(res, obj.id);
        for (const [key, entry] of obj.latest.entries()) {
            res.push(key);
            this.ts(res, entry.id);
            this.encodeNode(res, entry.node);
        }
    }
    encodeArr(arr, obj) {
        const res = [1];
        arr.push(res);
        this.ts(res, obj.id);
        for (const chunk of obj.chunks())
            this.encodeArrChunk(res, chunk);
    }
    encodeArrChunk(arr, chunk) {
        this.ts(arr, chunk.id);
        if (chunk.deleted)
            arr.push(chunk.deleted);
        else {
            const nodes = [];
            for (const n of chunk.nodes)
                this.encodeNode(nodes, n);
            arr.push(nodes);
        }
    }
    encodeStr(arr, obj) {
        const res = [2];
        arr.push(res);
        this.ts(res, obj.id);
        for (const chunk of obj.chunks())
            this.encodeStrChunk(res, chunk);
    }
    encodeStrChunk(arr, chunk) {
        this.ts(arr, chunk.id);
        if (chunk.deleted)
            arr.push(chunk.deleted);
        else
            arr.push(chunk.str);
    }
    encodeVal(arr, obj) {
        const res = [3];
        arr.push(res);
        this.ts(res, obj.id);
        this.ts(res, obj.writeId);
        res.push(obj.value);
    }
    encodeConst(arr, obj) {
        switch (obj.value) {
            case null: {
                arr.push(1);
                return;
            }
            case false: {
                arr.push(2);
                return;
            }
            case true: {
                arr.push(3);
                return;
            }
            case undefined: {
                arr.push(4);
                return;
            }
        }
        const res = [4];
        arr.push(res);
        res.push(obj.value);
    }
}
