import { ClockDecoder } from '../../../json-crdt-patch/codec/clock/ClockDecoder';
import { Model } from '../../model';
import { DocRootType } from '../../types/lww-doc-root/DocRootType';
import { ORIGIN } from '../../../json-crdt-patch/constants';
import { FALSE, NULL, TRUE, UNDEFINED } from '../../constants';
import { ConstantType } from '../../types/const/ConstantType';
import { ObjectChunk } from '../../types/lww-object/ObjectChunk';
import { ObjectType } from '../../types/lww-object/ObjectType';
import { ValueType } from '../../types/lww-value/ValueType';
import { ArrayChunk } from '../../types/rga-array/ArrayChunk';
import { ArrayType } from '../../types/rga-array/ArrayType';
import { StringChunk } from '../../types/rga-string/StringChunk';
import { StringType } from '../../types/rga-string/StringType';
export class LogicalDecoder {
    decode(data) {
        this.clockDecoder = ClockDecoder.fromArr(data[0]);
        const doc = Model.withLogicalClock(this.clockDecoder.clock);
        const rootId = this.ts(data, 1);
        const rootNode = data[3] ? this.decodeNode(doc, data[3]) : null;
        doc.root = new DocRootType(doc, rootId, rootNode);
        return doc;
    }
    ts(arr, index) {
        const sessionIndex = arr[index];
        const timeDiff = arr[index + 1];
        return this.clockDecoder.decodeId(sessionIndex, timeDiff);
    }
    decodeNode(doc, data) {
        switch (data) {
            case 1:
                return NULL;
            case 2:
                return FALSE;
            case 3:
                return TRUE;
            case 4:
                return UNDEFINED;
        }
        if (data instanceof Array) {
            switch (data[0]) {
                case 0:
                    return this.decodeObj(doc, data);
                case 1:
                    return this.decodeArr(doc, data);
                case 2:
                    return this.decodeStr(doc, data);
                case 3:
                    return this.decodeVal(doc, data);
                case 4:
                    return this.decodeConst(doc, data);
            }
        }
        throw new Error('UNKNOWN_NODE');
    }
    decodeObj(doc, data) {
        const id = this.ts(data, 1);
        const obj = new ObjectType(doc, id);
        const length = data.length;
        for (let i = 3; i < length; i += 4) {
            const key = data[i];
            const keyId = this.ts(data, i + 1);
            const node = this.decodeNode(doc, data[i + 3]);
            const chunk = new ObjectChunk(keyId, node);
            obj.putChunk(key, chunk);
        }
        doc.nodes.index(obj);
        return obj;
    }
    decodeArr(doc, data) {
        const id = this.ts(data, 1);
        const obj = new ArrayType(doc, id);
        const length = data.length;
        for (let i = 3; i < length; i += 3) {
            const chunkId = this.ts(data, i);
            const content = data[i + 2];
            let chunk;
            if (typeof content === 'number') {
                chunk = new ArrayChunk(chunkId, undefined);
                chunk.deleted = content;
            }
            else {
                const nodes = content.map((c) => this.decodeNode(doc, c));
                chunk = new ArrayChunk(chunkId, nodes);
            }
            obj.append(chunk);
        }
        doc.nodes.index(obj);
        return obj;
    }
    decodeStr(doc, data) {
        const id = this.ts(data, 1);
        const obj = new StringType(doc, id);
        const length = data.length;
        for (let i = 3; i < length; i += 3) {
            const chunkId = this.ts(data, i);
            const content = data[i + 2];
            let chunk;
            if (typeof content === 'number') {
                chunk = new StringChunk(chunkId, undefined);
                chunk.deleted = content;
            }
            else {
                chunk = new StringChunk(chunkId, content);
            }
            obj.append(chunk);
        }
        doc.nodes.index(obj);
        return obj;
    }
    decodeVal(doc, data) {
        const id = this.ts(data, 1);
        const writeId = this.ts(data, 3);
        const value = data[5];
        const obj = new ValueType(id, writeId, value);
        doc.nodes.index(obj);
        return obj;
    }
    decodeConst(doc, data) {
        const value = data[1];
        const obj = new ConstantType(ORIGIN, value);
        return obj;
    }
}
