import { ORIGIN } from '../../../json-crdt-patch/constants';
import { FALSE, NULL, TRUE, UNDEFINED } from '../../constants';
import { ConstantType } from '../../types/const/ConstantType';
import { DocRootType } from '../../types/lww-doc-root/DocRootType';
import { ObjectChunk } from '../../types/lww-object/ObjectChunk';
import { ObjectType } from '../../types/lww-object/ObjectType';
import { ValueType } from '../../types/lww-value/ValueType';
import { ArrayChunk } from '../../types/rga-array/ArrayChunk';
import { ArrayType } from '../../types/rga-array/ArrayType';
import { StringChunk } from '../../types/rga-string/StringChunk';
import { StringType } from '../../types/rga-string/StringType';
export class AbstractDecoder {
    decodeRoot(doc, { id, node }) {
        const ts = this.decodeTimestamp(id);
        const jsonNode = node ? this.decodeNode(doc, node) : null;
        const root = new DocRootType(doc, ts, jsonNode);
        doc.root = root;
    }
    decodeNode(doc, node) {
        switch (node.type) {
            case 'obj':
                return this.decodeObj(doc, node);
            case 'arr':
                return this.decodeArr(doc, node);
            case 'str':
                return this.decodeStr(doc, node);
            case 'val':
                return this.decodeVal(doc, node);
            case 'const':
                return this.decodeConst(doc, node);
        }
        throw new Error('UNKNOWN_NODE');
    }
    decodeObj(doc, node) {
        const id = this.decodeTimestamp(node.id);
        const obj = new ObjectType(doc, id);
        const keys = Object.keys(node.chunks);
        for (const key of keys) {
            const val = node.chunks[key];
            obj.putChunk(key, new ObjectChunk(this.decodeTimestamp(val.id), this.decodeNode(doc, val.node)));
        }
        doc.nodes.index(obj);
        return obj;
    }
    decodeArr(doc, node) {
        const obj = new ArrayType(doc, this.decodeTimestamp(node.id));
        for (const c of node.chunks)
            obj.append(this.decodeArrChunk(doc, c));
        doc.nodes.index(obj);
        return obj;
    }
    decodeArrChunk(doc, c) {
        const id = this.decodeTimestamp(c.id);
        if (typeof c.span === 'number') {
            const chunk = new ArrayChunk(id, undefined);
            chunk.deleted = c.span;
            return chunk;
        }
        else
            return new ArrayChunk(id, c.nodes.map((n) => this.decodeNode(doc, n)));
    }
    decodeStr(doc, node) {
        const obj = new StringType(doc, this.decodeTimestamp(node.id));
        for (const c of node.chunks)
            obj.append(this.decodeStrChunk(doc, c));
        doc.nodes.index(obj);
        return obj;
    }
    decodeStrChunk(doc, c) {
        const id = this.decodeTimestamp(c.id);
        if (typeof c.span === 'number') {
            const chunk = new StringChunk(id, undefined);
            chunk.deleted = c.span;
            return chunk;
        }
        else
            return new StringChunk(id, c.value);
    }
    decodeVal(doc, node) {
        const obj = new ValueType(this.decodeTimestamp(node.id), this.decodeTimestamp(node.writeId), node.value);
        doc.nodes.index(obj);
        return obj;
    }
    decodeConst(doc, node) {
        switch (node.value) {
            case null:
                return NULL;
            case true:
                return TRUE;
            case false:
                return FALSE;
            case undefined:
                return UNDEFINED;
        }
        return new ConstantType(ORIGIN, node.value);
    }
}
