import { ConstantType } from '../../types/const/ConstantType';
import { ObjectType } from '../../types/lww-object/ObjectType';
import { ValueType } from '../../types/lww-value/ValueType';
import { ArrayType } from '../../types/rga-array/ArrayType';
import { StringType } from '../../types/rga-string/StringType';
export class AbstractEncoder {
    encodeRoot(root) {
        return {
            type: 'root',
            id: this.encodeTimestamp(root.id),
            node: root.node ? this.encodeNode(root.node) : null,
        };
    }
    encodeNode(node) {
        if (node instanceof ObjectType)
            return this.encodeObj(node);
        else if (node instanceof ArrayType)
            return this.encodeArr(node);
        else if (node instanceof StringType)
            return this.encodeStr(node);
        else if (node instanceof ValueType)
            return this.encodeVal(node);
        else if (node instanceof ConstantType)
            return this.encodeConst(node);
        throw new Error('UNKNOWN_NODE');
    }
    encodeObj(obj) {
        const chunks = {};
        for (const [key, entry] of obj.latest.entries())
            chunks[key] = {
                id: this.encodeTimestamp(entry.id),
                node: this.encodeNode(entry.node),
            };
        return {
            type: 'obj',
            id: this.encodeTimestamp(obj.id),
            chunks,
        };
    }
    encodeArr(obj) {
        const chunks = [];
        for (const chunk of obj.chunks())
            chunks.push(this.encodeArrChunk(chunk));
        return {
            type: 'arr',
            id: this.encodeTimestamp(obj.id),
            chunks,
        };
    }
    encodeArrChunk(chunk) {
        if (chunk.deleted) {
            const tombstone = {
                id: this.encodeTimestamp(chunk.id),
                span: chunk.deleted,
            };
            return tombstone;
        }
        const res = {
            id: this.encodeTimestamp(chunk.id),
            nodes: chunk.nodes.map((n) => this.encodeNode(n)),
        };
        return res;
    }
    encodeStr(obj) {
        const chunks = [];
        for (const chunk of obj.chunks())
            chunks.push(this.encodeStrChunk(chunk));
        return {
            type: 'str',
            id: this.encodeTimestamp(obj.id),
            chunks,
        };
    }
    encodeStrChunk(chunk) {
        if (chunk.deleted) {
            const tombstone = {
                id: this.encodeTimestamp(chunk.id),
                span: chunk.deleted,
            };
            return tombstone;
        }
        const res = {
            id: this.encodeTimestamp(chunk.id),
            value: chunk.str,
        };
        return res;
    }
    encodeVal(obj) {
        return {
            type: 'val',
            id: this.encodeTimestamp(obj.id),
            writeId: this.encodeTimestamp(obj.writeId),
            value: obj.value,
        };
    }
    encodeConst(obj) {
        return {
            type: 'const',
            value: obj.value,
        };
    }
}
