import { OpAdd } from '../../op/OpAdd';
import { OpRemove } from '../../op/OpRemove';
import { OpReplace } from '../../op/OpReplace';
import { OpMove } from '../../op/OpMove';
import { OpCopy } from '../../op/OpCopy';
import { OpTest } from '../../op/OpTest';
import { OpFlip } from '../../op/OpFlip';
import { OpInc } from '../../op/OpInc';
import { OpStrIns } from '../../op/OpStrIns';
import { OpStrDel } from '../../op/OpStrDel';
import { OpSplit } from '../../op/OpSplit';
import { OpMerge } from '../../op/OpMerge';
import { OpExtend } from '../../op/OpExtend';
import { OpDefined } from '../../op/OpDefined';
import { OpUndefined } from '../../op/OpUndefined';
import { OpTestType } from '../../op/OpTestType';
import { OpTestString } from '../../op/OpTestString';
import { OpTestStringLen } from '../../op/OpTestStringLen';
import { OpContains } from '../../op/OpContains';
import { OpEnds } from '../../op/OpEnds';
import { OpStarts } from '../../op/OpStarts';
import { OpIn } from '../../op/OpIn';
import { OpLess } from '../../op/OpLess';
import { OpMore } from '../../op/OpMore';
import { OpAnd } from '../../op/OpAnd';
import { OpOr } from '../../op/OpOr';
import { OpNot } from '../../op/OpNot';
import { OpMatches } from '../../op/OpMatches';
import { OpType } from '../../op/OpType';
import { toPath } from '../../../json-pointer';
export const compactToOp = (op) => {
    switch (op[0]) {
        case 0: return new OpAdd(toPath(op[1]), op[2]);
        case 1: return new OpRemove(toPath(op[1]), op[2]);
        case 2: return new OpReplace(toPath(op[1]), op[2], op[3]);
        case 4: return new OpMove(toPath(op[1]), toPath(op[2]));
        case 3: return new OpCopy(toPath(op[1]), toPath(op[2]));
        case 8: return new OpFlip(toPath(op[1]));
        case 9: return new OpInc(toPath(op[1]), op[2]);
        case 6: return new OpStrIns(toPath(op[1]), op[2], op[3]);
        case 7: return new OpStrDel(toPath(op[1]), op[2], op[3] || undefined, op[4]);
        case 10: return new OpSplit(toPath(op[1]), op[2], op[3] || null);
        case 11: return new OpMerge(toPath(op[1]), op[2], op[3] || null);
        case 12: return new OpExtend(toPath(op[1]), op[2], !!op[3]);
        default: return compactToPredicateOp(op);
    }
};
export const compactToPredicateOp = (op) => {
    switch (op[0]) {
        case 5: return new OpTest(toPath(op[1]), op[2], !!op[3]);
        case 31: return new OpDefined(toPath(op[1]));
        case 38: return new OpUndefined(toPath(op[1]));
        case 42: return new OpType(toPath(op[1]), op[2]);
        case 39: return new OpTestType(toPath(op[1]), op[2]);
        case 40: return new OpTestString(toPath(op[1]), op[2], op[3], !!op[4]);
        case 41: return new OpTestStringLen(toPath(op[1]), op[2], !!op[3]);
        case 30: return new OpContains(toPath(op[1]), op[2], !!op[3]);
        case 32: return new OpEnds(toPath(op[1]), op[2], !!op[3]);
        case 37: return new OpStarts(toPath(op[1]), op[2], !!op[3]);
        case 35: return new OpMatches(toPath(op[1]), op[2], !!op[3]);
        case 33: return new OpIn(toPath(op[1]), op[2]);
        case 34: return new OpLess(toPath(op[1]), op[2]);
        case 36: return new OpMore(toPath(op[1]), op[2]);
        case 43: {
            const path = toPath(op[1]);
            return new OpAnd(path, op[2].map((x) => {
                const copy = [...x];
                copy[1] = [...path, ...toPath(x[1])];
                return compactToPredicateOp(copy);
            }));
        }
        case 45: {
            const path = toPath(op[1]);
            return new OpOr(path, op[2].map((x) => {
                const copy = [...x];
                copy[1] = [...path, ...toPath(x[1])];
                return compactToPredicateOp(copy);
            }));
        }
        case 44: {
            const path = toPath(op[1]);
            return new OpNot(path, op[2].map((x) => {
                const copy = [...x];
                copy[1] = [...path, ...toPath(x[1])];
                return compactToPredicateOp(copy);
            }));
        }
        default:
            throw new Error('OP_UNKNOWN');
    }
};
export function decode(patch) {
    const ops = [];
    const length = patch.length;
    for (let i = 0; i < length; i++) {
        const op = compactToOp(patch[i]);
        ops.push(op);
    }
    return ops;
}
