import type { OPCODE } from '../../constants';
import type { Path } from '../../../json-pointer/util';
import type { JsonPatchTypes } from '../json/types';
export declare type CompactOp = CompactJsonPatchOp | CompactJsonPredicateOp | CompactJsonPatchExtendedOp;
export declare type CompactOpBase = [opcode: OPCODE, path: string | Path] | [opcode: OPCODE, path: string | Path, arg1?: unknown] | [opcode: OPCODE, path: string | Path, arg1?: unknown, arg2?: unknown] | [opcode: OPCODE, path: string | Path, arg1?: unknown, arg2?: unknown, arg3?: unknown];
export declare type CompactJsonPatchOp = CompactAddOp | CompactCopyOp | CompactMoveOp | CompactRemoveOp | CompactReplaceOp | CompactTestOp;
export declare type CompactAddOp = [opcode: OPCODE.add, path: string | Path, value: unknown];
export declare type CompactCopyOp = [opcode: OPCODE.copy, path: string | Path, from: string | Path];
export declare type CompactMoveOp = [opcode: OPCODE.move, path: string | Path, from: string | Path];
export declare type CompactRemoveOp = [opcode: OPCODE.remove, path: string | Path] | [opcode: OPCODE.remove, path: string | Path, oldValue?: unknown];
export declare type CompactReplaceOp = [opcode: OPCODE.replace, path: Path | string, value: unknown] | [opcode: OPCODE.replace, path: Path | string, value: unknown, oldValue?: unknown];
export declare type CompactTestOp = [opcode: OPCODE.test, path: string | Path, value: unknown] | [opcode: OPCODE.test, path: string | Path, value: unknown, not?: 1];
export declare type CompactJsonPredicateOp = CompactAndOp | CompactContainsOp | CompactDefinedOp | CompactEndsOp | CompactInOp | CompactLessOp | CompactMatchesOp | CompactMoreOp | CompactNotOp | CompactOrOp | CompactStartsOp | CompactTestTypeOp | CompactTypeOp | CompactStartsOp | CompactUndefinedOp;
export declare type CompactAndOp = [opcode: OPCODE.and, path: string | Path, ops: CompactOpBase[]];
export declare type CompactContainsOp = [opcode: OPCODE.contains, path: string | Path, value: string] | [opcode: OPCODE.contains, path: string | Path, value: string, ignore_case?: 1];
export declare type CompactDefinedOp = [opcode: OPCODE.defined, path: string | Path];
export declare type CompactEndsOp = [opcode: OPCODE.ends, path: string | Path, value: string] | [opcode: OPCODE.ends, path: string | Path, value: string, ignore_case?: 1];
export declare type CompactInOp = [opcode: OPCODE.in, path: string | Path, value: unknown[]];
export declare type CompactLessOp = [opcode: OPCODE.less, path: string | Path, value: number];
export declare type CompactMatchesOp = [opcode: OPCODE.matches, path: string | Path, value: string] | [opcode: OPCODE.matches, path: string | Path, value: string, ignore_case?: 1];
export declare type CompactMoreOp = [opcode: OPCODE.more, path: string | Path, value: number];
export declare type CompactNotOp = [opcode: OPCODE.not, path: string | Path, ops: CompactOpBase[]];
export declare type CompactOrOp = [opcode: OPCODE.or, path: string | Path, ops: CompactOpBase[]];
export declare type CompactStartsOp = [opcode: OPCODE.starts, path: string | Path, value: string] | [opcode: OPCODE.starts, path: string | Path, value: string, ignore_case?: 1];
export declare type CompactTestTypeOp = [opcode: OPCODE.test_type, path: string | Path, type: JsonPatchTypes[]];
export declare type CompactTypeOp = [opcode: OPCODE.type, path: string | Path, value: JsonPatchTypes];
export declare type CompactUndefinedOp = [opcode: OPCODE.undefined, path: string | Path];
export declare type CompactJsonPatchExtendedOp = CompactExtendOp | CompactFlipOp | CompactIncOp | CompactMergeOp | CompactSplitOp | CompactStrDelOp | CompactStrInsOp | CompactTestStringOp | CompactTestStringLenOp;
export declare type CompactExtendOp = [opcode: OPCODE.extend, path: string | Path, props: Record<string, unknown>] | [opcode: OPCODE.extend, path: string | Path, props: Record<string, unknown>, deleteNull?: 1];
export declare type CompactFlipOp = [opcode: OPCODE.flip, path: string | Path];
export declare type CompactIncOp = [opcode: OPCODE.inc, path: string | Path, inc: number];
export declare type CompactMergeOp = [opcode: OPCODE.merge, path: string | Path, pos: number] | [opcode: OPCODE.merge, path: string | Path, pos: number, props?: object | null];
export declare type CompactSplitOp = [opcode: OPCODE.split, path: string | Path, pos: number] | [opcode: OPCODE.split, path: string | Path, pos: number, props?: object | null];
export declare type CompactStrDelOp = [opcode: OPCODE.str_del, path: string | Path, pos: number, str: string] | [opcode: OPCODE.str_del, path: string | Path, pos: number, str: 0, len: number];
export declare type CompactStrInsOp = [opcode: OPCODE.str_ins, path: string | Path, pos: number, str: string];
export declare type CompactTestStringOp = [opcode: OPCODE.test_string, path: string | Path, pos: number, str: string] | [opcode: OPCODE.test_string, path: string | Path, pos: number, str: string, not?: 1];
export declare type CompactTestStringLenOp = [opcode: OPCODE.test_string_len, path: string | Path, len: number] | [opcode: OPCODE.test_string_len, path: string | Path, len: number, not?: 1];
