import { AbstractOp } from './AbstractOp';
import { find, formatJsonPointer } from '../../json-pointer';
import { OpAdd } from './OpAdd';
import { deepClone } from '../util';
export class OpCopy extends AbstractOp {
    constructor(path, from) {
        super(path);
        this.from = from;
    }
    op() {
        return 'copy';
    }
    apply(doc) {
        const { val } = find(doc, this.from);
        if (val === undefined)
            throw new Error('NOT_FOUND');
        const add = new OpAdd(this.path, deepClone(val)).apply(doc);
        return add;
    }
    toJson(parent) {
        return {
            op: 'copy',
            path: formatJsonPointer(this.path),
            from: formatJsonPointer(this.from),
        };
    }
    toCompact(parent) {
        return [3, this.path, this.from];
    }
    encode(encoder, parent) {
        encoder.encodeArrayHeader(3);
        encoder.u8(3);
        encoder.encodeArray(this.path);
        encoder.encodeArray(this.from);
    }
}
