import { find, formatJsonPointer } from '../../json-pointer';
import { AbstractPredicateOp } from './AbstractPredicateOp';
const isEqual = require('fast-deep-equal');
export class OpTest extends AbstractPredicateOp {
    constructor(path, value, not) {
        super(path);
        this.value = value;
        this.not = not;
    }
    op() {
        return 'test';
    }
    test(doc) {
        const { val } = find(doc, this.path);
        if (val === undefined)
            return !!this.not;
        const test = isEqual(val, this.value);
        return this.not ? !test : test;
    }
    toJson(parent) {
        const op = {
            op: 'test',
            path: formatJsonPointer(parent ? this.path.slice(parent.path.length) : this.path),
            value: this.value,
        };
        if (this.not)
            op.not = this.not;
        return op;
    }
    toCompact(parent) {
        const path = parent ? this.path.slice(parent.path.length) : this.path;
        return this.not
            ? [5, path, this.value, 1]
            : [5, path, this.value];
    }
    encode(encoder, parent) {
        encoder.encodeArrayHeader(this.not ? 4 : 3);
        encoder.u8(5);
        encoder.encodeArray(parent ? this.path.slice(parent.path.length) : this.path);
        encoder.encodeAny(this.value);
        if (this.not)
            encoder.u8(1);
    }
}
