import { BinaryNotificationMessage, BinaryRequestDataMessage, BinaryRequestCompleteMessage, BinaryRequestErrorMessage, BinaryRequestUnsubscribeMessage, BinaryResponseDataMessage, BinaryResponseCompleteMessage, BinaryResponseErrorMessage, BinaryResponseUnsubscribeMessage, } from '../../messages/binary';
import { RequestDataMessage, RequestCompleteMessage, RequestErrorMessage, RequestUnsubscribeMessage, ResponseDataMessage, ResponseCompleteMessage, ResponseErrorMessage, ResponseUnsubscribeMessage, } from '../../messages/nominal';
import { Encoder as EncoderBinary } from '../binary/Encoder';
import { Encoder as MessagePackEncoder } from '../../../../json-pack/Encoder';
const EMPTY = new Uint8Array([]);
export class Encoder {
    constructor() {
        this.binary = new EncoderBinary();
        this.msgpack = new MessagePackEncoder();
    }
    encodeData(data) {
        if (data === undefined)
            return EMPTY;
        return this.msgpack.encode(data);
    }
    convertMessage(message) {
        if (message instanceof ResponseDataMessage)
            return new BinaryResponseDataMessage(message.id, this.encodeData(message.data));
        else if (message instanceof ResponseCompleteMessage)
            return new BinaryResponseCompleteMessage(message.id, this.encodeData(message.data));
        else if (message instanceof ResponseErrorMessage)
            return new BinaryResponseErrorMessage(message.id, this.encodeData(message.data));
        else if (message instanceof ResponseUnsubscribeMessage)
            return new BinaryResponseUnsubscribeMessage(message.id);
        else if (message instanceof RequestDataMessage)
            return new BinaryRequestDataMessage(message.id, message.method, this.encodeData(message.data));
        else if (message instanceof RequestCompleteMessage)
            return new BinaryRequestCompleteMessage(message.id, message.method, this.encodeData(message.data));
        else if (message instanceof RequestErrorMessage)
            return new BinaryRequestErrorMessage(message.id, message.method, this.encodeData(message.data));
        else if (message instanceof RequestUnsubscribeMessage)
            return new BinaryRequestUnsubscribeMessage(message.id);
        return new BinaryNotificationMessage(message.method, this.encodeData(message.data));
    }
    encode(messages) {
        const binaryMessages = [];
        const length = messages.length;
        for (let i = 0; i < length; i++)
            binaryMessages.push(this.convertMessage(messages[i]));
        return this.binary.encode(binaryMessages);
    }
}
