import { NotificationMessage, RequestCompleteMessage, RequestDataMessage, RequestErrorMessage, RequestUnsubscribeMessage, ResponseCompleteMessage, ResponseDataMessage, ResponseErrorMessage, ResponseUnsubscribeMessage } from '../../messages/nominal';
export function decodeMsg(message) {
    if (!(message instanceof Array)) {
        throw new Error('Invalid message');
    }
    const length = message.length;
    if (length < 1 || length > 4) {
        throw new Error('Invalid message');
    }
    const first = message[0];
    if (typeof first === 'number') {
        switch (first) {
            case 0: {
                const [, id, data] = message;
                return new ResponseCompleteMessage(id, data);
            }
            case -1: {
                const [, id, data] = message;
                return new ResponseErrorMessage(id, data);
            }
            case -2: {
                const [, id, data] = message;
                return new ResponseDataMessage(id, data);
            }
            case -3: {
                const [, id] = message;
                return new ResponseUnsubscribeMessage(id);
            }
            default: {
                const [, second] = message;
                switch (second) {
                    case 0: {
                        const [, , name, data] = message;
                        return new RequestDataMessage(first, name, data);
                    }
                    case 1: {
                        const [, , method, data] = message;
                        return new RequestErrorMessage(first, method, data);
                    }
                    case 2: {
                        return new RequestUnsubscribeMessage(first);
                    }
                    default: {
                        const [, name, data] = message;
                        return new RequestCompleteMessage(first, name, data);
                    }
                }
            }
        }
    }
    const [method, data] = message;
    return new NotificationMessage(method, data);
}
export function decode(messages) {
    if (messages[0] instanceof Array) {
        const length = messages.length;
        const out = [];
        for (let i = 0; i < length; i++)
            out.push(decodeMsg(messages[i]));
        return out;
    }
    else
        return decodeMsg(messages);
}
