import { Observable, Observer } from 'rxjs';
import type { IRpcApiCaller, RpcMethod, RpcMethodRequest, RpcMethodResponse } from './types';
import { ErrorFormatter } from './error';
export interface RpcApiCallerParams<Api extends Record<string, RpcMethod<Ctx, any, any>>, Ctx = unknown, E = unknown> {
    api: Api;
    error?: ErrorFormatter<E>;
    preCallBufferSize?: number;
    maxActiveCalls?: number;
}
export interface Call<Request = unknown, Response = unknown> {
    req$: Observer<Request>;
    reqUnsubscribe$: Observable<null>;
    res$: Observable<Response>;
}
export declare class RpcApiCaller<Api extends Record<string, RpcMethod<Ctx, any, any>>, Ctx = unknown, E = unknown> implements IRpcApiCaller<Api, Ctx> {
    readonly api: Api;
    readonly error: ErrorFormatter<E>;
    protected readonly preCallBufferSize: number;
    protected readonly maxActiveCalls: number;
    protected _calls: number;
    constructor({ api, error, preCallBufferSize, maxActiveCalls, }: RpcApiCallerParams<Api, Ctx, E>);
    get activeCalls(): number;
    exists<K extends keyof Api>(name: K): boolean;
    get<K extends keyof Api>(name: K): Api[K];
    call<K extends keyof Api>(name: K, request: RpcMethodRequest<Api[K]>, ctx: Ctx): Promise<RpcMethodResponse<Api[K]>>;
    createCall<K extends keyof Api>(name: K, ctx: Ctx): Call<RpcMethodRequest<Api[K]>, RpcMethodResponse<Api[K]>>;
    call$<K extends keyof Api>(name: K, request$: Observable<RpcMethodRequest<Api[K]>>, ctx: Ctx): Observable<RpcMethodResponse<Api[K]>>;
}
