import { ReplaySubject } from 'rxjs';
import { filter, switchMap, takeUntil } from 'rxjs/operators';
import { RpcClient } from '../rpc';
import { RpcDuplex } from '../rpc/RpcDuplex';
import { RpcServer } from '../rpc/RpcServer';
import { PersistentChannel } from '../channel/channel';
export class RpcPersistentChannel {
    constructor(params) {
        this.rpc$ = new ReplaySubject(1);
        this.channel = new PersistentChannel(params.channel);
        this.channel.open$.pipe(filter(open => open)).subscribe(() => {
            const close$ = this.channel.open$.pipe(filter(open => !open));
            const rpc = new RpcDuplex({
                client: new RpcClient({
                    ...params.client,
                    send: (messages) => {
                        const encoded = params.codec.encoder.encode(messages);
                        this.channel.send$(encoded).subscribe();
                    },
                }),
                server: new RpcServer({
                    ...params.server,
                    send: (messages) => {
                        const encoded = params.codec.encoder.encode(messages);
                        this.channel.send$(encoded).subscribe();
                    },
                }),
            });
            this.channel.message$
                .pipe(takeUntil(close$))
                .subscribe(data => {
                const encoded = typeof data === 'string' ? data : new Uint8Array(data);
                const messages = params.codec.decoder.decode(encoded);
                rpc.onMessages(messages, {});
            });
            if (this.rpc)
                this.rpc.disconnect();
            this.rpc = rpc;
            this.rpc$.next(rpc);
        });
    }
    start() {
        this.channel.start();
    }
    stop() {
        this.rpc?.stop();
        this.channel.stop();
    }
    call$(method, data) {
        return this.rpc$
            .pipe(switchMap(rpc => rpc.call$(method, data)));
    }
    notify(method, data) {
        this.rpc$
            .subscribe(rpc => rpc.notify(method, data));
    }
}
