const EMPTY_UINT8 = new Uint8Array([]);
const EMPTY_VIEW = new DataView(EMPTY_UINT8.buffer);
export class Encoder {
    constructor() {
        this.uint8 = EMPTY_UINT8;
        this.view = EMPTY_VIEW;
        this.offset = 0;
    }
    grow(size) {
        const newUint8 = new Uint8Array(size);
        newUint8.set(this.uint8);
        this.uint8 = newUint8;
        this.view = new DataView(newUint8.buffer);
    }
    ensureCapacity(capacity) {
        const size = this.offset + capacity;
        const len = this.uint8.byteLength;
        if (len < size)
            this.grow(Math.max(size, len * 4));
    }
    reset() {
        this.uint8 = new Uint8Array(1024);
        this.view = new DataView(this.uint8.buffer);
        this.offset = 0;
    }
    flush() {
        return this.uint8.subarray(0, this.offset);
    }
    u8(char) {
        this.ensureCapacity(1);
        this.uint8[this.offset++] = char;
    }
    u16(word) {
        this.ensureCapacity(2);
        this.view.setUint16(this.offset, word);
        this.offset += 2;
    }
    u32(dword) {
        this.ensureCapacity(4);
        this.view.setUint32(this.offset, dword);
        this.offset += 4;
    }
}
