"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Encoder = void 0;
var header_1 = require("./header");
var DataMessage_1 = require("../messages/DataMessage");
var NotificationMessage_1 = require("../messages/NotificationMessage");
var SubscribeMessage_1 = require("../messages/SubscribeMessage");
var CompleteMessage_1 = require("../messages/CompleteMessage");
var ErrorMessage_1 = require("../messages/ErrorMessage");
var UnsubscribeMessage_1 = require("../messages/UnsubscribeMessage");
var Encoder = (function () {
    function Encoder() {
        this.uint8 = new Uint8Array(0);
        this.offset = 0;
    }
    Encoder.prototype.encode = function (messages) {
        var size = 0;
        var length = messages.length;
        for (var i = 0; i < length; i++)
            size += messages[i].size();
        this.uint8 = new Uint8Array(size);
        this.offset = 0;
        for (var i = 0; i < length; i++) {
            var message = messages[i];
            if (message instanceof NotificationMessage_1.NotificationMessage)
                this.notif(message);
            else if (message instanceof SubscribeMessage_1.SubscribeMessage)
                this.sub(message);
            else if (message instanceof DataMessage_1.DataMessage)
                this.data(message);
            else if (message instanceof CompleteMessage_1.CompleteMessage)
                this.comp(message);
            else if (message instanceof ErrorMessage_1.ErrorMessage)
                this.err(message);
            else if (message instanceof UnsubscribeMessage_1.UnsubscribeMessage)
                this.unsub(message);
        }
        return this.uint8;
    };
    Encoder.prototype.notif = function (message) {
        var method = message.method, data = message.data;
        var length = data ? data.byteLength : 0;
        this.header(0, length);
        this.method(method);
        if (length) {
            this.uint8.set(data, this.offset);
            this.offset += length;
        }
    };
    Encoder.prototype.sub = function (message) {
        var id = message.id, method = message.method, data = message.data;
        var length = data ? data.byteLength : 0;
        this.header(1, length);
        this.id(id);
        this.method(method);
        if (length) {
            this.uint8.set(data, this.offset);
            this.offset += length;
        }
    };
    Encoder.prototype.data = function (message) {
        this.msgWithId(2, message.id, message.data);
    };
    Encoder.prototype.comp = function (message) {
        this.msgWithId(3, message.id, message.data);
    };
    Encoder.prototype.err = function (message) {
        this.msgWithId(5, message.id, message.data);
    };
    Encoder.prototype.unsub = function (message) {
        this.uint8[this.offset++] = 4 << 5;
        this.id(message.id);
    };
    Encoder.prototype.msgWithId = function (code, id, data) {
        var length = data ? data.byteLength : 0;
        this.header(code, length);
        this.id(id);
        if (length) {
            this.uint8.set(data, this.offset);
            this.offset += length;
        }
    };
    Encoder.prototype.header = function (code, length) {
        this.offset = header_1.writeHeader(this.uint8, this.offset, code, length);
    };
    Encoder.prototype.id = function (id) {
        this.uint8[this.offset++] = id >>> 8;
        this.uint8[this.offset++] = id & 0xff;
    };
    Encoder.prototype.method = function (method) {
        var uint8 = this.uint8;
        var length = method.length;
        var offset = this.offset;
        uint8[offset++] = length;
        for (var i = 0; i < length; i++)
            uint8[offset++] = method.charCodeAt(i);
        this.offset = offset;
    };
    return Encoder;
}());
exports.Encoder = Encoder;
