"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decodeFullMessages = exports.decodeFullMessage = void 0;
var tslib_1 = require("tslib");
var CompleteMessage_1 = require("../messages/CompleteMessage");
var DataMessage_1 = require("../messages/DataMessage");
var ErrorMessage_1 = require("../messages/ErrorMessage");
var NotificationMessage_1 = require("../messages/NotificationMessage");
var SubscribeMessage_1 = require("../messages/SubscribeMessage");
var UnsubscribeMessage_1 = require("../messages/UnsubscribeMessage");
var header_1 = require("./header");
var readMethod = function (arr, offset) {
    var size = arr[offset++];
    var str = '';
    for (var i = 0; i < size; i++)
        str += String.fromCharCode(arr[offset++]);
    return [str, offset];
};
var decodeFullMessage = function (arr, offset) {
    var byte1 = arr[offset++];
    var code = byte1 >>> 5;
    switch (code) {
        case 0: {
            var _a = tslib_1.__read(header_1.readHeader(byte1, arr, offset), 2), length_1 = _a[0], off1 = _a[1];
            var _b = tslib_1.__read(readMethod(arr, off1), 2), method = _b[0], off2 = _b[1];
            var data = arr.subarray(off2, off2 + length_1);
            return [new NotificationMessage_1.NotificationMessage(method, data), off2 + length_1];
        }
        case 1: {
            var _c = tslib_1.__read(header_1.readHeader(byte1, arr, offset), 2), length_2 = _c[0], off1 = _c[1];
            var o1 = arr[off1];
            var o2 = arr[off1 + 1];
            var id = (o1 << 8) | o2;
            var _d = tslib_1.__read(readMethod(arr, off1 + 2), 2), method = _d[0], off2 = _d[1];
            var data = arr.subarray(off2, off2 + length_2);
            return [new SubscribeMessage_1.SubscribeMessage(id, method, data), off2 + length_2];
        }
        case 2:
        case 3:
        case 5: {
            var _e = tslib_1.__read(header_1.readHeader(byte1, arr, offset), 2), length_3 = _e[0], off1 = _e[1];
            var o1 = arr[off1];
            var o2 = arr[off1 + 1];
            var id = (o1 << 8) | o2;
            offset = off1 + 2;
            var data = arr.subarray(offset, offset + length_3);
            var message = code === 2
                ? new DataMessage_1.DataMessage(id, data)
                : code === 3
                    ? new CompleteMessage_1.CompleteMessage(id, data)
                    : new ErrorMessage_1.ErrorMessage(id, data);
            return [message, offset + length_3];
        }
        case 4: {
            var o1 = arr[offset++];
            var o2 = arr[offset++];
            var id = (o1 << 8) | o2;
            return [new UnsubscribeMessage_1.UnsubscribeMessage(id), offset];
        }
    }
    throw new Error('UNKNOWN_MESSAGE');
};
exports.decodeFullMessage = decodeFullMessage;
var decodeFullMessages = function (arr, offset) {
    var messages = [];
    var length = arr.length;
    while (offset < length) {
        var _a = tslib_1.__read(exports.decodeFullMessage(arr, offset), 2), message = _a[0], off = _a[1];
        offset = off;
        messages.push(message);
    }
    return messages;
};
exports.decodeFullMessages = decodeFullMessages;
