"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.readHeader = exports.writeHeader = exports.getHeaderSize = void 0;
var getHeaderSize = function (length) {
    if (length <= 15)
        return 1;
    if (length <= 2047)
        return 2;
    if (length <= 262143)
        return 3;
    return 4;
};
exports.getHeaderSize = getHeaderSize;
var writeHeader = function (arr, offset, type, length) {
    if (length <= 15) {
        arr[offset++] = (type << 5) | length;
        return offset;
    }
    if (length <= 2047) {
        arr[offset++] = (type << 5) | 16 | (15 & length);
        arr[offset++] = length >>> 4;
        return offset;
    }
    if (length <= 262143) {
        arr[offset++] = (type << 5) | 16 | (15 & length);
        arr[offset++] = 128 | ((length >>> 4) & 0xff);
        arr[offset++] = length >>> 11;
        return offset;
    }
    if (length <= 33554431) {
        arr[offset++] = (type << 5) | 16 | (15 & length);
        arr[offset++] = 128 | ((length >>> 4) & 0xff);
        arr[offset++] = 128 | ((length >>> 11) & 0xff);
        arr[offset++] = length >>> 18;
        return offset;
    }
    throw new Error('LENGTH_TOO_BIG');
};
exports.writeHeader = writeHeader;
var readHeader = function (byte1, arr, offset) {
    var more = byte1 & 16;
    var value = byte1 & 15;
    if (!more)
        return [value, offset];
    var octet = arr[offset++];
    more = octet & 128;
    value = (octet << 4) | value;
    if (!more)
        return [value, offset];
    octet = arr[offset++];
    more = octet & 128;
    value = (octet << 11) | value;
    if (!more)
        return [value, offset];
    octet = arr[offset++];
    value = (octet << 18) | value;
    return [value, offset];
};
exports.readHeader = readHeader;
