"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServerVectorClock = exports.ServerClock = exports.ServerTimespan = exports.ServerTimestamp = void 0;
var tslib_1 = require("tslib");
var SERVER_SESSION_ID = 1;
var ServerTimestamp = (function () {
    function ServerTimestamp(time) {
        this.time = time;
    }
    ServerTimestamp.prototype.getSessionId = function () {
        return SERVER_SESSION_ID;
    };
    ServerTimestamp.prototype.isEqual = function (ts) {
        return this.getSessionId() === ts.getSessionId() && this.time === ts.time;
    };
    ServerTimestamp.prototype.compare = function (ts) {
        if (this.time > ts.time)
            return 1;
        if (this.time < ts.time)
            return -1;
        if (this.getSessionId() > ts.getSessionId())
            return 1;
        if (this.getSessionId() < ts.getSessionId())
            return -1;
        return 0;
    };
    ServerTimestamp.prototype.inSpan = function (span, ts, tsSpan) {
        if (this.getSessionId() !== ts.getSessionId())
            return false;
        if (this.time > ts.time)
            return false;
        if (this.time + span < ts.time + tsSpan)
            return false;
        return true;
    };
    ServerTimestamp.prototype.overlap = function (span, ts, tsSpan) {
        if (this.getSessionId() !== ts.getSessionId())
            return 0;
        var x1 = this.time;
        var x2 = x1 + span;
        var y1 = ts.time;
        var y2 = y1 + tsSpan;
        var min = Math.max(x1, y1);
        var max = Math.min(x2, y2);
        var diff = max - min;
        return diff <= 0 ? 0 : diff;
    };
    ServerTimestamp.prototype.tick = function (cycles) {
        return new ServerTimestamp(this.time + cycles);
    };
    ServerTimestamp.prototype.interval = function (cycles, span) {
        return new ServerTimespan(this.time + cycles, span);
    };
    ServerTimestamp.prototype.toString = function () {
        return this.getSessionId() + '!' + this.time;
    };
    ServerTimestamp.prototype.toDisplayString = function () {
        var session = String(this.getSessionId());
        if (session.length > 4)
            session = '..' + session.substr(session.length - 4);
        return session + '!' + this.time;
    };
    ServerTimestamp.prototype.clock = function () {
        return new ServerClock(this.time);
    };
    ServerTimestamp.prototype.compact = function () {
        return this.getSessionId() + ',' + this.time;
    };
    return ServerTimestamp;
}());
exports.ServerTimestamp = ServerTimestamp;
var ServerTimespan = (function (_super) {
    tslib_1.__extends(ServerTimespan, _super);
    function ServerTimespan(time, span) {
        var _this = _super.call(this, time) || this;
        _this.span = span;
        return _this;
    }
    return ServerTimespan;
}(ServerTimestamp));
exports.ServerTimespan = ServerTimespan;
var ServerClock = (function (_super) {
    tslib_1.__extends(ServerClock, _super);
    function ServerClock() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ServerClock.prototype.tick = function (cycles) {
        var timestamp = new ServerTimestamp(this.time);
        this.time += cycles;
        return timestamp;
    };
    return ServerClock;
}(ServerTimestamp));
exports.ServerClock = ServerClock;
var ServerVectorClock = (function (_super) {
    tslib_1.__extends(ServerVectorClock, _super);
    function ServerVectorClock() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.clocks = new Map();
        return _this;
    }
    ServerVectorClock.prototype.observe = function (ts, span) {
        if (ts.getSessionId() !== SERVER_SESSION_ID)
            throw new Error('INVALID_SERVER_SESSION');
        if (this.time < ts.time)
            throw new Error('TIME_TRAVEL');
        var time = ts.time + span;
        if (time > this.time)
            this.time = time;
    };
    ServerVectorClock.prototype.clone = function () {
        return this.fork();
    };
    ServerVectorClock.prototype.fork = function () {
        return new ServerVectorClock(this.time);
    };
    return ServerVectorClock;
}(ServerClock));
exports.ServerVectorClock = ServerVectorClock;
