"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decode = exports.decodeTimestamp = void 0;
var decodeString_1 = require("../../../util/decodeString");
var clock_1 = require("../../clock");
var PatchBuilder_1 = require("../../PatchBuilder");
var varuint_1 = require("./util/varuint");
var decodeTimestamp = function (buf, offset) {
    var o1 = buf[offset];
    var o2 = buf[offset + 1];
    var o3 = buf[offset + 2];
    var o4 = buf[offset + 3];
    var o5 = buf[offset + 4];
    var o6 = buf[offset + 5];
    var o7 = buf[offset + 6];
    var o8 = buf[offset + 7];
    var sessionId = o8;
    sessionId *= 0x100;
    sessionId += o4;
    sessionId *= 0x100;
    sessionId += o3;
    sessionId *= 0x100;
    sessionId += o2;
    sessionId *= 0x100;
    sessionId += o1;
    var time = o7;
    time *= 0x100;
    time += o6;
    time *= 0x100;
    time += o5;
    return new clock_1.LogicalTimestamp(sessionId, time);
};
exports.decodeTimestamp = decodeTimestamp;
var decode = function (buf) {
    var id = exports.decodeTimestamp(buf, 0);
    var offset = 8;
    var clock = new clock_1.LogicalVectorClock(id.getSessionId(), id.time);
    var builder = new PatchBuilder_1.PatchBuilder(clock);
    var length = buf.byteLength;
    var ts = function () {
        var value = exports.decodeTimestamp(buf, offset);
        offset += 8;
        return value;
    };
    var varuint = function () {
        var value = varuint_1.decodeVarUint(buf, offset);
        offset +=
            value <= 127 ? 1 : value <= 32767 ? 2 : value <= 8388607 ? 3 : 4;
        return value;
    };
    while (offset < length) {
        var opcode = buf[offset];
        offset++;
        switch (opcode) {
            case 0: {
                builder.obj();
                continue;
            }
            case 1: {
                builder.arr();
                continue;
            }
            case 2: {
                builder.str();
                continue;
            }
            case 3: {
                builder.num();
                continue;
            }
            case 4: {
                builder.root(ts());
                continue;
            }
            case 5: {
                var object = ts();
                var fields = varuint();
                var tuples = [];
                for (var i = 0; i < fields; i++) {
                    var value = ts();
                    var strLength = varuint();
                    var key = decodeString_1.decodeString(buf, offset, strLength);
                    offset += strLength;
                    tuples.push([key, value]);
                }
                builder.setKeys(object, tuples);
                continue;
            }
            case 6: {
                var after = ts();
                var value = new Float64Array(buf.slice(offset, offset + 8).buffer)[0];
                offset += 8;
                builder.setNum(after, value);
                continue;
            }
            case 7: {
                var obj = ts();
                var after = ts();
                var length_1 = varuint();
                var str = decodeString_1.decodeString(buf, offset, length_1);
                offset += length_1;
                builder.insStr(obj, after, str);
                continue;
            }
            case 8: {
                var arr = ts();
                var after = ts();
                var length_2 = varuint();
                var elements = [];
                for (var i = 0; i < length_2; i++) {
                    var value = ts();
                    elements.push(value);
                }
                builder.insArr(arr, after, elements);
                continue;
            }
            case 9: {
                var obj = ts();
                var after = ts();
                var length_3 = varuint();
                builder.del(obj, after, length_3);
                continue;
            }
            case 10: {
                var obj = ts();
                var after = ts();
                builder.del(obj, after, 1);
                continue;
            }
            case 11: {
                builder.noop(1);
                continue;
            }
            case 12: {
                builder.noop(varuint());
                continue;
            }
        }
    }
    return builder.patch;
};
exports.decode = decode;
