"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.encode = exports.encodeTimestamp = void 0;
var tslib_1 = require("tslib");
var encodeString_1 = require("../../../util/encodeString");
var DeleteOperation_1 = require("../../operations/DeleteOperation");
var InsertArrayElementsOperation_1 = require("../../operations/InsertArrayElementsOperation");
var InsertStringSubstringOperation_1 = require("../../operations/InsertStringSubstringOperation");
var MakeArrayOperation_1 = require("../../operations/MakeArrayOperation");
var MakeNumberOperation_1 = require("../../operations/MakeNumberOperation");
var MakeObjectOperation_1 = require("../../operations/MakeObjectOperation");
var MakeStringOperation_1 = require("../../operations/MakeStringOperation");
var NoopOperation_1 = require("../../operations/NoopOperation");
var SetNumberOperation_1 = require("../../operations/SetNumberOperation");
var SetObjectKeysOperation_1 = require("../../operations/SetObjectKeysOperation");
var SetRootOperation_1 = require("../../operations/SetRootOperation");
var varuint_1 = require("./util/varuint");
var encodeTimestamp = function (ts) {
    var sessionId = ts.getSessionId();
    var low32 = sessionId | 0;
    if (low32 < 0)
        low32 += 4294967296;
    var high8 = (sessionId - low32) / 4294967296;
    return [low32, ((high8 << 24) | ts.time) >>> 0];
};
exports.encodeTimestamp = encodeTimestamp;
var encode = function (patch) {
    var e_1, _a, e_2, _b, e_3, _c, e_4, _d;
    var ops = patch.ops;
    var buffers = [new Uint32Array(exports.encodeTimestamp(patch.getId())).buffer];
    var size = 8;
    try {
        for (var ops_1 = tslib_1.__values(ops), ops_1_1 = ops_1.next(); !ops_1_1.done; ops_1_1 = ops_1.next()) {
            var op = ops_1_1.value;
            if (op instanceof MakeObjectOperation_1.MakeObjectOperation) {
                buffers.push(new Uint8Array([0]));
                size += 1;
                continue;
            }
            if (op instanceof MakeArrayOperation_1.MakeArrayOperation) {
                buffers.push(new Uint8Array([1]));
                size += 1;
                continue;
            }
            if (op instanceof MakeStringOperation_1.MakeStringOperation) {
                buffers.push(new Uint8Array([2]));
                size += 1;
                continue;
            }
            if (op instanceof MakeNumberOperation_1.MakeNumberOperation) {
                buffers.push(new Uint8Array([3]));
                size += 1;
                continue;
            }
            if (op instanceof SetRootOperation_1.SetRootOperation) {
                buffers.push(new Uint8Array([4]));
                buffers.push(new Uint32Array(exports.encodeTimestamp(op.value)).buffer);
                size += 1 + 8;
                continue;
            }
            if (op instanceof SetObjectKeysOperation_1.SetObjectKeysOperation) {
                buffers.push(new Uint8Array([5]));
                buffers.push(new Uint32Array(exports.encodeTimestamp(op.object)).buffer);
                size += 1 + 8;
                var keyNumberBuffer = new Uint8Array(varuint_1.encodeVarUInt(op.tuples.length));
                buffers.push(keyNumberBuffer);
                size += keyNumberBuffer.byteLength;
                try {
                    for (var _e = (e_2 = void 0, tslib_1.__values(op.tuples)), _f = _e.next(); !_f.done; _f = _e.next()) {
                        var _g = tslib_1.__read(_f.value, 2), key = _g[0], value = _g[1];
                        var valueBuffer = new Uint32Array(exports.encodeTimestamp(value)).buffer;
                        var keyBuffer = encodeString_1.encodeString(key);
                        var keyLengthBuffer = new Uint8Array(varuint_1.encodeVarUInt(keyBuffer.byteLength));
                        buffers.push(valueBuffer, keyLengthBuffer, keyBuffer);
                        size += valueBuffer.byteLength + keyLengthBuffer.byteLength + keyBuffer.byteLength;
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (_f && !_f.done && (_b = _e.return)) _b.call(_e);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
                continue;
            }
            if (op instanceof SetNumberOperation_1.SetNumberOperation) {
                buffers.push(new Uint8Array([6]), new Uint32Array(exports.encodeTimestamp(op.num)).buffer, new Float64Array([op.value]).buffer);
                size += 1 + 8 + 8;
                continue;
            }
            if (op instanceof InsertStringSubstringOperation_1.InsertStringSubstringOperation) {
                var stringBuffer = encodeString_1.encodeString(op.substring);
                var stringLengthBuffer = new Uint8Array(varuint_1.encodeVarUInt(stringBuffer.byteLength));
                buffers.push(new Uint8Array([7]), new Uint32Array(exports.encodeTimestamp(op.obj)).buffer, new Uint32Array(exports.encodeTimestamp(op.after)).buffer, stringLengthBuffer.buffer, stringBuffer);
                size += 1 + 8 + 8 + stringLengthBuffer.byteLength + stringBuffer.byteLength;
                continue;
            }
            if (op instanceof InsertArrayElementsOperation_1.InsertArrayElementsOperation) {
                var arr = op.arr, after = op.after, elements = op.elements;
                var length_1 = elements.length;
                var elementLengthBuffer = new Uint8Array(varuint_1.encodeVarUInt(length_1));
                buffers.push(new Uint8Array([8]), new Uint32Array(exports.encodeTimestamp(arr)).buffer, new Uint32Array(exports.encodeTimestamp(after)).buffer, elementLengthBuffer.buffer);
                try {
                    for (var elements_1 = (e_3 = void 0, tslib_1.__values(elements)), elements_1_1 = elements_1.next(); !elements_1_1.done; elements_1_1 = elements_1.next()) {
                        var element = elements_1_1.value;
                        buffers.push(new Uint32Array(exports.encodeTimestamp(element)).buffer);
                    }
                }
                catch (e_3_1) { e_3 = { error: e_3_1 }; }
                finally {
                    try {
                        if (elements_1_1 && !elements_1_1.done && (_c = elements_1.return)) _c.call(elements_1);
                    }
                    finally { if (e_3) throw e_3.error; }
                }
                size += 1 + 8 + 8 + elementLengthBuffer.byteLength + 8 * length_1;
                continue;
            }
            if (op instanceof DeleteOperation_1.DeleteOperation) {
                var obj = op.obj, after = op.after, length_2 = op.length;
                if (length_2 > 1) {
                    var spanBuffer = new Uint8Array(varuint_1.encodeVarUInt(length_2));
                    buffers.push(new Uint8Array([9]), new Uint32Array(exports.encodeTimestamp(obj)).buffer, new Uint32Array(exports.encodeTimestamp(after)).buffer, spanBuffer.buffer);
                    size += 1 + 8 + 8 + spanBuffer.byteLength;
                    continue;
                }
                buffers.push(new Uint8Array([10]), new Uint32Array(exports.encodeTimestamp(obj)).buffer, new Uint32Array(exports.encodeTimestamp(after)).buffer);
                size += 1 + 8 + 8;
                continue;
            }
            if (op instanceof NoopOperation_1.NoopOperation) {
                var length_3 = op.length;
                if (length_3 > 1) {
                    var spanBuffer = new Uint8Array(varuint_1.encodeVarUInt(length_3));
                    buffers.push(new Uint8Array([12]), spanBuffer.buffer);
                    size += 1 + spanBuffer.byteLength;
                    continue;
                }
                buffers.push(new Uint8Array([11]));
                size += 1;
                continue;
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (ops_1_1 && !ops_1_1.done && (_a = ops_1.return)) _a.call(ops_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    var res = new Uint8Array(size);
    var offset = 0;
    try {
        for (var buffers_1 = tslib_1.__values(buffers), buffers_1_1 = buffers_1.next(); !buffers_1_1.done; buffers_1_1 = buffers_1.next()) {
            var buffer = buffers_1_1.value;
            res.set(new Uint8Array(buffer), offset);
            offset += buffer.byteLength;
        }
    }
    catch (e_4_1) { e_4 = { error: e_4_1 }; }
    finally {
        try {
            if (buffers_1_1 && !buffers_1_1.done && (_d = buffers_1.return)) _d.call(buffers_1);
        }
        finally { if (e_4) throw e_4.error; }
    }
    return res;
};
exports.encode = encode;
