"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.varUint8Size = exports.decodeVarUint8 = exports.writeVarUint8 = exports.encodeVarUint8Buf = exports.encodeVarUint8 = void 0;
var encodeVarUint8 = function (uint) {
    if (uint <= 127)
        return [uint];
    if (uint <= 16383)
        return [128 | (uint & 127), (uint & 16256) >> 7];
    if (uint <= 2097151)
        return [
            128 | (uint & 127),
            128 | ((uint >> 7) & 127),
            (uint & 2080768) >> 14,
        ];
    if (uint <= 268435455)
        return [
            128 | (uint & 127),
            128 | ((uint >> 7) & 127),
            128 | ((uint >> 14) & 127),
            (uint & 266338304) >> 21,
        ];
    if (uint <= 34359738367) {
        var lo32_1 = uint | 0;
        if (lo32_1 < 0)
            lo32_1 += 4294967296;
        var hi32_1 = (uint - lo32_1) / 4294967296;
        return [
            128 | (uint & 127),
            128 | ((uint >> 7) & 127),
            128 | ((uint >> 14) & 127),
            128 | ((uint >> 21) & 127),
            (hi32_1 << 4) | (uint >>> 28),
        ];
    }
    var lo32 = uint | 0;
    if (lo32 < 0)
        lo32 += 4294967296;
    var hi32 = (uint - lo32) / 4294967296;
    if (uint <= 4398046511103)
        return [
            128 | (uint & 127),
            128 | ((uint >> 7) & 127),
            128 | ((uint >> 14) & 127),
            128 | ((uint >> 21) & 127),
            128 | ((hi32 & 7) << 4) | (uint >>> 28),
            hi32 >>> 3,
        ];
    if (uint <= 562949953421311)
        return [
            128 | (uint & 127),
            128 | ((uint >> 7) & 127),
            128 | ((uint >> 14) & 127),
            128 | ((uint >> 21) & 127),
            128 | ((hi32 & 7) << 4) | (uint >>> 28),
            128 | ((hi32 & 1016) >>> 3),
            hi32 >>> 10,
        ];
    return [
        128 | (uint & 127),
        128 | ((uint >> 7) & 127),
        128 | ((uint >> 14) & 127),
        128 | ((uint >> 21) & 127),
        128 | ((hi32 & 7) << 4) | (uint >>> 28),
        128 | ((hi32 & 1016) >>> 3),
        128 | ((hi32 & 130048) >>> 10),
        hi32 >>> 17,
    ];
};
exports.encodeVarUint8 = encodeVarUint8;
var encodeVarUint8Buf = function (uint) { return new Uint8Array(exports.encodeVarUint8(uint)); };
exports.encodeVarUint8Buf = encodeVarUint8Buf;
var writeVarUint8 = function (num, uint8, offset) {
    var octets = exports.encodeVarUint8(num);
    for (var i = 0; i < octets.length; i++)
        uint8[offset++] = octets[i];
    return offset;
};
exports.writeVarUint8 = writeVarUint8;
var decodeVarUint8 = function (buf, offset) {
    var o1 = buf[offset++];
    if (o1 <= 127)
        return [o1, offset];
    var o2 = buf[offset++];
    if (o2 <= 127)
        return [(o2 << 7) | (o1 & 127), offset];
    var o3 = buf[offset++];
    if (o3 <= 127)
        return [(o3 << 14) | ((o2 & 127) << 7) | (o1 & 127), offset];
    var o4 = buf[offset++];
    if (o4 <= 127)
        return [(o4 << 21) | ((o3 & 127) << 14) | ((o2 & 127) << 7) | (o1 & 127), offset];
    var o5 = buf[offset++];
    if (o5 <= 127)
        return [
            o5 * 268435456 +
                (((o4 & 127) << 21) | ((o3 & 127) << 14) | ((o2 & 127) << 7) | (o1 & 127)),
            offset,
        ];
    var o6 = buf[offset++];
    if (o6 <= 127)
        return [
            o6 * 34359738368 +
                ((o5 & 127) * 268435456 +
                    (((o4 & 127) << 21) | ((o3 & 127) << 14) | ((o2 & 127) << 7) | (o1 & 127))),
            offset,
        ];
    var o7 = buf[offset++];
    if (o7 <= 127)
        return [
            o7 * 4398046511104 +
                ((o6 & 127) * 34359738368 +
                    ((o5 & 127) * 268435456 +
                        (((o4 & 127) << 21) | ((o3 & 127) << 14) | ((o2 & 127) << 7) | (o1 & 127)))),
            offset,
        ];
    var o8 = buf[offset++];
    return [
        o8 * 562949953421312 +
            ((o7 & 127) * 4398046511104 +
                ((o6 & 127) * 34359738368 +
                    ((o5 & 127) * 268435456 +
                        (((o4 & 127) << 21) | ((o3 & 127) << 14) | ((o2 & 127) << 7) | (o1 & 127))))),
        offset,
    ];
};
exports.decodeVarUint8 = decodeVarUint8;
var varUint8Size = function (uint) {
    if (uint <= 127)
        return 1;
    if (uint <= 16383)
        return 2;
    if (uint <= 2097151)
        return 3;
    if (uint <= 268435455)
        return 4;
    if (uint <= 34359738367)
        return 5;
    if (uint <= 4398046511103)
        return 6;
    if (uint <= 562949953421311)
        return 7;
    return 8;
};
exports.varUint8Size = varUint8Size;
