"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClockDecoder = void 0;
var clock_1 = require("../../clock");
var ClockDecoder = (function () {
    function ClockDecoder(sessionId, time) {
        this.table = new Map();
        this.index = 1;
        this.clock = new clock_1.LogicalVectorClock(sessionId, time);
        if (time)
            this.clock.observe(new clock_1.LogicalTimestamp(sessionId, time), 1);
        this.table.set(this.index++, this.clock);
    }
    ClockDecoder.fromArr = function (arr) {
        var decoder = new ClockDecoder(arr[0], arr[1]);
        var length = arr.length;
        for (var i = 2; i < length; i += 2)
            decoder.pushTuple(arr[i], arr[i + 1]);
        return decoder;
    };
    ClockDecoder.prototype.pushTuple = function (sessionId, time) {
        var ts = new clock_1.LogicalTimestamp(sessionId, time);
        this.clock.observe(ts, 1);
        this.table.set(this.index++, ts);
    };
    ClockDecoder.prototype.decodeId = function (sessionIndex, timeDiff) {
        if (!sessionIndex)
            return new clock_1.LogicalTimestamp(0, timeDiff);
        var ts = this.table.get(sessionIndex);
        if (!ts)
            throw new Error('INVALID_CLOCK_TABLE');
        return new clock_1.LogicalTimestamp(ts.getSessionId(), ts.time - timeDiff);
    };
    return ClockDecoder;
}());
exports.ClockDecoder = ClockDecoder;
