"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClockEncoder = void 0;
var tslib_1 = require("tslib");
var RelativeTimestamp_1 = require("./RelativeTimestamp");
var ClockEncoder = (function () {
    function ClockEncoder(clock) {
        this.clock = clock;
        this.index = 1;
        this.table = new Map();
        this.table.set(clock.getSessionId(), this.index++);
    }
    ClockEncoder.prototype.append = function (ts) {
        var time = ts.time;
        var sessionId = ts.getSessionId();
        if (sessionId === 0)
            return new RelativeTimestamp_1.RelativeTimestamp(0, ts.time);
        var clock = this.clock.clocks.get(sessionId);
        if (!clock)
            throw new Error("Clock not found (" + sessionId + ", " + time + ").");
        if (!this.table.has(sessionId))
            this.table.set(sessionId, this.index++);
        var sessionIndex = this.table.get(sessionId);
        var timeDiff = clock.time - time;
        var relativeTimestamp = new RelativeTimestamp_1.RelativeTimestamp(sessionIndex, timeDiff);
        return relativeTimestamp;
    };
    ClockEncoder.prototype.toJson = function () {
        var e_1, _a;
        var out = [];
        try {
            for (var _b = tslib_1.__values(this.table.keys()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var sessionId = _c.value;
                var clock = this.clock.clocks.get(sessionId);
                if (clock)
                    out.push(clock.getSessionId(), clock.time);
                else if (this.clock.getSessionId() === sessionId) {
                    out.push(sessionId, this.clock.time);
                }
                else {
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return out;
    };
    ClockEncoder.prototype.clocks = function () {
        var _a, _b, sessionId, e_2_1;
        var e_2, _c;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    _d.trys.push([0, 5, 6, 7]);
                    _a = tslib_1.__values(this.table.keys()), _b = _a.next();
                    _d.label = 1;
                case 1:
                    if (!!_b.done) return [3, 4];
                    sessionId = _b.value;
                    return [4, this.clock.clocks.get(sessionId)];
                case 2:
                    _d.sent();
                    _d.label = 3;
                case 3:
                    _b = _a.next();
                    return [3, 1];
                case 4: return [3, 7];
                case 5:
                    e_2_1 = _d.sent();
                    e_2 = { error: e_2_1 };
                    return [3, 7];
                case 6:
                    try {
                        if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
                    }
                    finally { if (e_2) throw e_2.error; }
                    return [7];
                case 7: return [2];
            }
        });
    };
    return ClockEncoder;
}());
exports.ClockEncoder = ClockEncoder;
