"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.encode = void 0;
var tslib_1 = require("tslib");
var DeleteOperation_1 = require("../../operations/DeleteOperation");
var InsertArrayElementsOperation_1 = require("../../operations/InsertArrayElementsOperation");
var InsertStringSubstringOperation_1 = require("../../operations/InsertStringSubstringOperation");
var MakeArrayOperation_1 = require("../../operations/MakeArrayOperation");
var MakeConstantOperation_1 = require("../../operations/MakeConstantOperation");
var MakeNumberOperation_1 = require("../../operations/MakeNumberOperation");
var MakeObjectOperation_1 = require("../../operations/MakeObjectOperation");
var MakeStringOperation_1 = require("../../operations/MakeStringOperation");
var MakeValueOperation_1 = require("../../operations/MakeValueOperation");
var NoopOperation_1 = require("../../operations/NoopOperation");
var SetNumberOperation_1 = require("../../operations/SetNumberOperation");
var SetObjectKeysOperation_1 = require("../../operations/SetObjectKeysOperation");
var SetRootOperation_1 = require("../../operations/SetRootOperation");
var SetValueOperation_1 = require("../../operations/SetValueOperation");
var encode = function (patch) {
    var e_1, _a, e_2, _b, e_3, _c;
    var id = patch.getId();
    if (!id)
        throw new Error('PATCH_EMPTY');
    var sessionId = id.getSessionId();
    var time = id.time;
    var res = [sessionId, time];
    var pushTimestamp = function (ts) {
        var tsSessionId = ts.getSessionId();
        if (tsSessionId === sessionId)
            res.push(time - ts.time - 1);
        else
            res.push(tsSessionId, ts.time);
    };
    try {
        for (var _d = tslib_1.__values(patch.ops), _e = _d.next(); !_e.done; _e = _d.next()) {
            var op = _e.value;
            if (op instanceof MakeObjectOperation_1.MakeObjectOperation) {
                res.push(0);
                continue;
            }
            if (op instanceof MakeArrayOperation_1.MakeArrayOperation) {
                res.push(1);
                continue;
            }
            if (op instanceof MakeStringOperation_1.MakeStringOperation) {
                res.push(2);
                continue;
            }
            if (op instanceof MakeNumberOperation_1.MakeNumberOperation) {
                res.push(3);
                continue;
            }
            if (op instanceof SetRootOperation_1.SetRootOperation) {
                var value = op.value;
                res.push(4);
                pushTimestamp(value);
                continue;
            }
            if (op instanceof SetObjectKeysOperation_1.SetObjectKeysOperation) {
                var object = op.object, tuples = op.tuples;
                res.push(5, tuples.length);
                pushTimestamp(object);
                try {
                    for (var tuples_1 = (e_2 = void 0, tslib_1.__values(tuples)), tuples_1_1 = tuples_1.next(); !tuples_1_1.done; tuples_1_1 = tuples_1.next()) {
                        var _f = tslib_1.__read(tuples_1_1.value, 2), key = _f[0], value = _f[1];
                        res.push(key);
                        pushTimestamp(value);
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (tuples_1_1 && !tuples_1_1.done && (_b = tuples_1.return)) _b.call(tuples_1);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
                continue;
            }
            if (op instanceof SetNumberOperation_1.SetNumberOperation) {
                var num = op.num, value = op.value;
                res.push(6, value);
                pushTimestamp(num);
                continue;
            }
            if (op instanceof InsertStringSubstringOperation_1.InsertStringSubstringOperation) {
                var obj = op.obj, after = op.after, substring = op.substring;
                res.push(7, substring);
                pushTimestamp(obj);
                pushTimestamp(after);
                continue;
            }
            if (op instanceof InsertArrayElementsOperation_1.InsertArrayElementsOperation) {
                var arr = op.arr, after = op.after, elements = op.elements;
                res.push(8, elements.length);
                pushTimestamp(arr);
                pushTimestamp(after);
                try {
                    for (var elements_1 = (e_3 = void 0, tslib_1.__values(elements)), elements_1_1 = elements_1.next(); !elements_1_1.done; elements_1_1 = elements_1.next()) {
                        var element = elements_1_1.value;
                        pushTimestamp(element);
                    }
                }
                catch (e_3_1) { e_3 = { error: e_3_1 }; }
                finally {
                    try {
                        if (elements_1_1 && !elements_1_1.done && (_c = elements_1.return)) _c.call(elements_1);
                    }
                    finally { if (e_3) throw e_3.error; }
                }
                continue;
            }
            if (op instanceof DeleteOperation_1.DeleteOperation) {
                var obj = op.obj, after = op.after, length_1 = op.length;
                if (length_1 === 1) {
                    res.push(9);
                    pushTimestamp(obj);
                    pushTimestamp(after);
                }
                else {
                    res.push(10, length_1);
                    pushTimestamp(obj);
                    pushTimestamp(after);
                }
                continue;
            }
            if (op instanceof NoopOperation_1.NoopOperation) {
                var length_2 = op.length;
                if (length_2 === 1)
                    res.push(11);
                else
                    res.push(12, length_2);
                continue;
            }
            if (op instanceof MakeConstantOperation_1.MakeConstantOperation) {
                res.push(13);
                res.push(op.value);
                continue;
            }
            if (op instanceof MakeValueOperation_1.MakeValueOperation) {
                res.push(14);
                res.push(op.value);
                continue;
            }
            if (op instanceof SetValueOperation_1.SetValueOperation) {
                res.push(15);
                pushTimestamp(op.obj);
                res.push(op.value);
                continue;
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_e && !_e.done && (_a = _d.return)) _a.call(_d);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return res;
};
exports.encode = encode;
