"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decode = void 0;
var tslib_1 = require("tslib");
var clock_1 = require("../../clock");
var PatchBuilder_1 = require("../../PatchBuilder");
var ts = function (time) { return new clock_1.LogicalTimestamp(time[0], time[1]); };
var decode = function (data) {
    var e_1, _a;
    var id = data.id, ops = data.ops;
    var clock = new clock_1.LogicalVectorClock(id[0], id[1]);
    var builder = new PatchBuilder_1.PatchBuilder(clock);
    try {
        for (var ops_1 = tslib_1.__values(ops), ops_1_1 = ops_1.next(); !ops_1_1.done; ops_1_1 = ops_1.next()) {
            var op = ops_1_1.value;
            switch (op.op) {
                case 'obj': {
                    builder.obj();
                    break;
                }
                case 'arr': {
                    builder.arr();
                    break;
                }
                case 'str': {
                    builder.str();
                    break;
                }
                case 'num': {
                    builder.num();
                    break;
                }
                case 'val': {
                    builder.val(op.value);
                    break;
                }
                case 'const': {
                    builder.const(op.value);
                    break;
                }
                case 'root': {
                    builder.root(ts(op.value));
                    break;
                }
                case 'obj_set': {
                    builder.setKeys(ts(op.obj), op.tuples.map(function (_a) {
                        var _b = tslib_1.__read(_a, 2), key = _b[0], id = _b[1];
                        return [key, ts(id)];
                    }));
                    break;
                }
                case 'val_set': {
                    builder.setVal(ts(op.obj), op.value);
                    break;
                }
                case 'num_set': {
                    builder.setNum(ts(op.after), op.value);
                    break;
                }
                case 'str_ins': {
                    builder.insStr(ts(op.obj), ts(op.after || op.obj), op.value);
                    break;
                }
                case 'arr_ins': {
                    builder.insArr(ts(op.obj), ts(op.after || op.obj), op.values.map(ts));
                    break;
                }
                case 'del': {
                    builder.del(ts(op.obj), ts(op.after), op.len || 1);
                    break;
                }
                case 'noop': {
                    builder.noop(op.len || 1);
                    break;
                }
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (ops_1_1 && !ops_1_1.done && (_a = ops_1.return)) _a.call(ops_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return builder.patch;
};
exports.decode = decode;
