"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractDecoder = void 0;
var tslib_1 = require("tslib");
var constants_1 = require("../../../json-crdt-patch/constants");
var Decoder_1 = require("../../../json-pack/Decoder");
var constants_2 = require("../../constants");
var ConstantType_1 = require("../../types/const/ConstantType");
var DocRootType_1 = require("../../types/lww-doc-root/DocRootType");
var ObjectChunk_1 = require("../../types/lww-object/ObjectChunk");
var ObjectType_1 = require("../../types/lww-object/ObjectType");
var ValueType_1 = require("../../types/lww-value/ValueType");
var ArrayChunk_1 = require("../../types/rga-array/ArrayChunk");
var ArrayType_1 = require("../../types/rga-array/ArrayType");
var StringChunk_1 = require("../../types/rga-string/StringChunk");
var StringType_1 = require("../../types/rga-string/StringType");
var AbstractDecoder = (function (_super) {
    tslib_1.__extends(AbstractDecoder, _super);
    function AbstractDecoder() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    AbstractDecoder.prototype.decodeRoot = function (doc) {
        var id = this.ts();
        var node = this.x >= this.uint8.byteLength ? null : this.decodeNode();
        doc.root = new DocRootType_1.DocRootType(doc, id, node);
    };
    AbstractDecoder.prototype.decodeNode = function () {
        var byte = this.u8();
        if (byte < 128)
            return this.createConst(byte);
        else if (byte <= 143)
            return this.decodeObj(byte & 15);
        else if (byte <= 159)
            return this.decodeArr(byte & 15);
        else if (byte <= 191)
            return this.decodeStr(byte & 31);
        else if (byte >= 224)
            return this.createConst(byte - 0x100);
        else {
            switch (byte) {
                case 0xc0:
                    return constants_2.NULL;
                case 0xc1:
                    return constants_2.UNDEFINED;
                case 0xc2:
                    return constants_2.FALSE;
                case 0xc3:
                    return constants_2.TRUE;
                case 0xca:
                    return this.createConst(this.f32());
                case 0xcb:
                    return this.createConst(this.f64());
                case 0xcc:
                    return this.createConst(this.u8());
                case 0xcd:
                    return this.createConst(this.u16());
                case 0xce:
                    return this.createConst(this.u32());
                case 0xcf:
                    return this.createConst(this.u32() * 4294967296 + this.u32());
                case 0xd0:
                    return this.createConst(this.i8());
                case 0xd1:
                    return this.createConst(this.i16());
                case 0xd2:
                    return this.createConst(this.i32());
                case 0xd3:
                    return this.createConst(this.i32() * 4294967296 + this.i32());
                case 0xd5:
                    return this.decodeVal();
                case 0xde:
                    return this.decodeObj(this.u16());
                case 0xdf:
                    return this.decodeObj(this.u32());
                case 0xdc:
                    return this.decodeArr(this.u16());
                case 0xdd:
                    return this.decodeArr(this.u32());
                case 0xd9:
                    return this.decodeStr(this.u8());
                case 0xda:
                    return this.decodeStr(this.u16());
                case 0xdb:
                    return this.decodeStr(this.u32());
            }
        }
        return constants_2.NULL;
    };
    AbstractDecoder.prototype.createConst = function (value) {
        return new ConstantType_1.ConstantType(constants_1.ORIGIN, value);
    };
    AbstractDecoder.prototype.decodeObj = function (length) {
        var id = this.ts();
        var obj = new ObjectType_1.ObjectType(this.doc, id);
        for (var i = 0; i < length; i++)
            this.decodeObjChunk(obj);
        this.doc.nodes.index(obj);
        return obj;
    };
    AbstractDecoder.prototype.decodeObjChunk = function (obj) {
        var id = this.ts();
        var length = this.vuint57();
        var key = this.str(length);
        var node = this.decodeNode();
        var chunk = new ObjectChunk_1.ObjectChunk(id, node);
        obj.putChunk(key, chunk);
    };
    AbstractDecoder.prototype.decodeArr = function (length) {
        var id = this.ts();
        var obj = new ArrayType_1.ArrayType(this.doc, id);
        for (var i = 0; i < length; i++)
            this.decodeArrChunk(obj);
        this.doc.nodes.index(obj);
        return obj;
    };
    AbstractDecoder.prototype.decodeArrChunk = function (obj) {
        var _a = tslib_1.__read(this.b1vuint56(), 2), deleted = _a[0], length = _a[1];
        var id = this.ts();
        if (deleted) {
            var chunk = new ArrayChunk_1.ArrayChunk(id, undefined);
            chunk.deleted = length;
            obj.append(chunk);
        }
        else {
            var nodes = [];
            for (var i = 0; i < length; i++)
                nodes.push(this.decodeNode());
            var chunk = new ArrayChunk_1.ArrayChunk(id, nodes);
            obj.append(chunk);
        }
    };
    AbstractDecoder.prototype.decodeStr = function (length) {
        var id = this.ts();
        var obj = new StringType_1.StringType(this.doc, id);
        for (var i = 0; i < length; i++)
            this.decodeStrChunk(obj);
        this.doc.nodes.index(obj);
        return obj;
    };
    AbstractDecoder.prototype.decodeStrChunk = function (obj) {
        var _a = tslib_1.__read(this.b1vuint56(), 2), deleted = _a[0], length = _a[1];
        var id = this.ts();
        if (deleted) {
            var chunk = new StringChunk_1.StringChunk(id, undefined);
            chunk.deleted = length;
            obj.append(chunk);
        }
        else {
            var text = this.str(length);
            var chunk = new StringChunk_1.StringChunk(id, text);
            obj.append(chunk);
        }
    };
    AbstractDecoder.prototype.decodeVal = function () {
        var id = this.ts();
        var writeId = this.ts();
        var value = this.val();
        var obj = new ValueType_1.ValueType(id, writeId, value);
        this.doc.nodes.index(obj);
        return obj;
    };
    AbstractDecoder.prototype.id = function () {
        var byte = this.u8();
        if (byte <= 127)
            return [byte >>> 4, byte & 15];
        this.back(1);
        return [this.b1vuint28()[1], this.vuint39()];
    };
    AbstractDecoder.prototype.vuint57 = function () {
        var o1 = this.u8();
        if (o1 <= 127)
            return o1;
        var o2 = this.u8();
        if (o2 <= 127)
            return (o2 << 7) | (o1 & 127);
        var o3 = this.u8();
        if (o3 <= 127)
            return (o3 << 14) | ((o2 & 127) << 7) | (o1 & 127);
        var o4 = this.u8();
        if (o4 <= 127)
            return (o4 << 21) | ((o3 & 127) << 14) | ((o2 & 127) << 7) | (o1 & 127);
        var o5 = this.u8();
        if (o5 <= 127)
            return (o5 * 268435456 +
                (((o4 & 127) << 21) | ((o3 & 127) << 14) | ((o2 & 127) << 7) | (o1 & 127)));
        var o6 = this.u8();
        if (o6 <= 127)
            return (o6 * 34359738368 +
                ((o5 & 127) * 268435456 +
                    (((o4 & 127) << 21) | ((o3 & 127) << 14) | ((o2 & 127) << 7) | (o1 & 127))));
        var o7 = this.u8();
        if (o7 <= 127)
            return (o7 * 4398046511104 +
                ((o6 & 127) * 34359738368 +
                    ((o5 & 127) * 268435456 +
                        (((o4 & 127) << 21) | ((o3 & 127) << 14) | ((o2 & 127) << 7) | (o1 & 127)))));
        var o8 = this.u8();
        return (o8 * 562949953421312 +
            ((o7 & 127) * 4398046511104 +
                ((o6 & 127) * 34359738368 +
                    ((o5 & 127) * 268435456 +
                        (((o4 & 127) << 21) | ((o3 & 127) << 14) | ((o2 & 127) << 7) | (o1 & 127))))));
    };
    AbstractDecoder.prototype.vuint39 = function () {
        var o1 = this.u8();
        if (o1 <= 127)
            return o1;
        var o2 = this.u8();
        if (o2 <= 127)
            return (o2 << 7) | (o1 & 127);
        var o3 = this.u8();
        if (o3 <= 127)
            return (o3 << 14) | ((o2 & 127) << 7) | (o1 & 127);
        var o4 = this.u8();
        if (o4 <= 127)
            return (o4 << 21) | ((o3 & 127) << 14) | ((o2 & 127) << 7) | (o1 & 127);
        var o5 = this.u8();
        if (o5 <= 127)
            return (o5 * 268435456 +
                (((o4 & 127) << 21) | ((o3 & 127) << 14) | ((o2 & 127) << 7) | (o1 & 127)));
        var o6 = this.u8();
        return ((o6 & 15) * 34359738368 +
            ((o5 & 127) * 268435456 +
                (((o4 & 127) << 21) | ((o3 & 127) << 14) | ((o2 & 127) << 7) | (o1 & 127))));
    };
    AbstractDecoder.prototype.b1vuint56 = function () {
        var byte = this.u8();
        var flag = !!(byte & 128);
        var o1 = 127 & byte;
        if (o1 <= 63)
            return [flag, o1];
        var o2 = this.u8();
        if (o2 <= 127)
            return [flag, (o2 << 6) | (o1 & 63)];
        var o3 = this.u8();
        if (o3 <= 127)
            return [flag, (o3 << 13) | ((o2 & 127) << 6) | (o1 & 63)];
        var o4 = this.u8();
        if (o4 <= 127)
            return [flag, (o4 << 20) | ((o3 & 127) << 13) | ((o2 & 127) << 6) | (o1 & 63)];
        var o5 = this.u8();
        if (o5 <= 127)
            return [
                flag,
                o5 * 134217728 +
                    (((o4 & 127) << 20) | ((o3 & 127) << 13) | ((o2 & 127) << 6) | (o1 & 63)),
            ];
        var o6 = this.u8();
        if (o6 <= 127)
            return [
                flag,
                o6 * 17179869184 +
                    ((o5 & 127) * 134217728 +
                        (((o4 & 127) << 20) | ((o3 & 127) << 13) | ((o2 & 127) << 6) | (o1 & 63))),
            ];
        var o7 = this.u8();
        if (o7 <= 127)
            return [
                flag,
                o7 * 2199023255552 +
                    ((o6 & 127) * 17179869184 +
                        ((o5 & 127) * 134217728 +
                            (((o4 & 127) << 20) |
                                ((o3 & 127) << 13) |
                                ((o2 & 127) << 6) |
                                (o1 & 63)))),
            ];
        var o8 = this.u8();
        return [
            flag,
            o8 * 281474976710656 +
                ((o7 & 127) * 2199023255552 +
                    ((o6 & 127) * 17179869184 +
                        ((o5 & 127) * 134217728 +
                            (((o4 & 127) << 20) |
                                ((o3 & 127) << 13) |
                                ((o2 & 127) << 6) |
                                (o1 & 63))))),
        ];
    };
    AbstractDecoder.prototype.b1vuint28 = function () {
        var byte = this.u8();
        var flag = !!(byte & 128);
        var o1 = 127 & byte;
        if (o1 <= 63)
            return [flag, o1];
        var o2 = this.u8();
        if (o2 <= 127)
            return [flag, (o2 << 6) | (o1 & 63)];
        var o3 = this.u8();
        if (o3 <= 127)
            return [flag, (o3 << 13) | ((o2 & 127) << 6) | (o1 & 63)];
        var o4 = this.u8();
        return [flag, (o4 << 20) | ((o3 & 127) << 13) | ((o2 & 127) << 6) | (o1 & 63)];
    };
    AbstractDecoder.prototype.uint53vuint39 = function () {
        var x32 = this.u32();
        var x16 = this.u16();
        var y = this.u8();
        var z8 = this.u8();
        var x = (((y >>> 3) << 16) | x16) * 0x100000000 + x32;
        var z = ((y & 3) << 8) | z8;
        if (!(y & 4))
            return [x, z];
        var o1 = this.u8();
        if (o1 <= 127)
            return [x, (o1 << 10) | z];
        var o2 = this.u8();
        if (o2 <= 127)
            return [x, (o2 << 17) | ((o1 & 127) << 10) | z];
        var o3 = this.u8();
        if (o3 <= 127)
            return [x, (o3 << 24) | (((o2 & 127) << 17) | ((o1 & 127) << 10) | z)];
        var o4 = this.u8();
        return [
            x,
            o4 * 2147483648 +
                (((o3 & 127) << 24) | (((o2 & 127) << 17) | ((o1 & 127) << 10) | z)),
        ];
    };
    return AbstractDecoder;
}(Decoder_1.Decoder));
exports.AbstractDecoder = AbstractDecoder;
