"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractEncoder = void 0;
var tslib_1 = require("tslib");
var Encoder_1 = require("../../../json-pack/Encoder");
var utf8_1 = require("../../../util/utf8");
var ConstantType_1 = require("../../types/const/ConstantType");
var ObjectType_1 = require("../../types/lww-object/ObjectType");
var ValueType_1 = require("../../types/lww-value/ValueType");
var ArrayType_1 = require("../../types/rga-array/ArrayType");
var StringType_1 = require("../../types/rga-string/StringType");
var AbstractEncoder = (function (_super) {
    tslib_1.__extends(AbstractEncoder, _super);
    function AbstractEncoder() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    AbstractEncoder.prototype.encodeRoot = function (root) {
        this.ts(root.id);
        if (root.node)
            this.encodeNode(root.node);
    };
    AbstractEncoder.prototype.encodeNode = function (node) {
        if (node instanceof ObjectType_1.ObjectType)
            return this.encodeObj(node);
        else if (node instanceof ArrayType_1.ArrayType)
            return this.encodeArr(node);
        else if (node instanceof StringType_1.StringType)
            return this.encodeStr(node);
        else if (node instanceof ValueType_1.ValueType)
            return this.encodeVal(node);
        else if (node instanceof ConstantType_1.ConstantType)
            return this.encodeConst(node);
        throw new Error('UNKNOWN_NODE');
    };
    AbstractEncoder.prototype.encodeObj = function (obj) {
        var e_1, _a;
        var length = obj.latest.size;
        this.encodeObjectHeader(length);
        this.ts(obj.id);
        try {
            for (var _b = tslib_1.__values(obj.latest.entries()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = tslib_1.__read(_c.value, 2), key = _d[0], chunk = _d[1];
                this.encodeObjChunk(key, chunk);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    AbstractEncoder.prototype.encodeObjChunk = function (key, chunk) {
        this.ts(chunk.id);
        var length = utf8_1.utf8Count(key);
        this.vuint57(length);
        this.encodeUtf8(key, length);
        this.encodeNode(chunk.node);
    };
    AbstractEncoder.prototype.encodeArr = function (obj) {
        var e_2, _a;
        var length = obj.size();
        this.encodeArrayHeader(length);
        this.ts(obj.id);
        try {
            for (var _b = tslib_1.__values(obj.chunks()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var chunk = _c.value;
                this.encodeArrChunk(chunk);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
    };
    AbstractEncoder.prototype.encodeArrChunk = function (chunk) {
        if (chunk.deleted) {
            this.b1vuint56(true, chunk.deleted);
            this.ts(chunk.id);
        }
        else {
            var nodes = chunk.nodes;
            var length_1 = nodes.length;
            this.b1vuint56(false, length_1);
            this.ts(chunk.id);
            for (var i = 0; i < length_1; i++)
                this.encodeNode(nodes[i]);
        }
    };
    AbstractEncoder.prototype.encodeStr = function (obj) {
        var e_3, _a;
        var length = obj.size();
        this.encodeStringHeader(length);
        this.ts(obj.id);
        try {
            for (var _b = tslib_1.__values(obj.chunks()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var chunk = _c.value;
                this.encodeStrChunk(chunk);
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_3) throw e_3.error; }
        }
    };
    AbstractEncoder.prototype.encodeStrChunk = function (chunk) {
        if (chunk.deleted) {
            this.b1vuint56(true, chunk.deleted);
            this.ts(chunk.id);
        }
        else {
            var text = chunk.str;
            var length_2 = utf8_1.utf8Count(text);
            this.b1vuint56(false, length_2);
            this.ts(chunk.id);
            this.encodeUtf8(text, length_2);
        }
    };
    AbstractEncoder.prototype.encodeConst = function (obj) {
        var value = obj.value;
        switch (value) {
            case null:
            case undefined:
            case false:
            case true:
                this.encodeAny(value);
                break;
            default: {
                if (typeof value === 'number')
                    this.encodeNumber(value);
                else {
                    this.u8(0xd4);
                    this.encodeAny(value);
                }
            }
        }
    };
    AbstractEncoder.prototype.encodeVal = function (obj) {
        this.u8(0xd5);
        this.ts(obj.id);
        this.ts(obj.writeId);
        this.encodeAny(obj.value);
    };
    AbstractEncoder.prototype.encodeUtf8 = function (str, byteLength) {
        this.ensureCapacity(byteLength);
        var strLength = str.length;
        var uint8 = this.uint8;
        var offset = this.offset;
        var pos = 0;
        while (pos < strLength) {
            var value = str.charCodeAt(pos++);
            if ((value & 0xffffff80) === 0) {
                uint8[offset++] = value;
                continue;
            }
            else if ((value & 0xfffff800) === 0) {
                uint8[offset++] = ((value >> 6) & 0x1f) | 0xc0;
            }
            else {
                if (value >= 0xd800 && value <= 0xdbff) {
                    if (pos < strLength) {
                        var extra = str.charCodeAt(pos);
                        if ((extra & 0xfc00) === 0xdc00) {
                            pos++;
                            value = ((value & 0x3ff) << 10) + (extra & 0x3ff) + 0x10000;
                        }
                    }
                }
                if ((value & 0xffff0000) === 0) {
                    uint8[offset++] = ((value >> 12) & 0x0f) | 0xe0;
                    uint8[offset++] = ((value >> 6) & 0x3f) | 0x80;
                }
                else {
                    uint8[offset++] = ((value >> 18) & 0x07) | 0xf0;
                    uint8[offset++] = ((value >> 12) & 0x3f) | 0x80;
                    uint8[offset++] = ((value >> 6) & 0x3f) | 0x80;
                }
            }
            uint8[offset++] = (value & 0x3f) | 0x80;
        }
        this.offset = offset;
    };
    AbstractEncoder.prototype.id = function (x, y) {
        if (x <= 7 && y <= 15) {
            this.u8((x << 4) | y);
        }
        else {
            this.b1vuint28(true, x);
            this.vuint39(y);
        }
    };
    AbstractEncoder.prototype.vuint57 = function (num) {
        if (num <= 127) {
            this.u8(num);
        }
        else if (num <= 16383) {
            this.u8(128 | (num & 127));
            this.u8(num >>> 7);
        }
        else if (num <= 2097151) {
            this.u8(128 | (num & 127));
            this.u8(128 | ((num >>> 7) & 127));
            this.u8(num >>> 14);
        }
        else if (num <= 268435455) {
            this.u8(128 | (num & 127));
            this.u8(128 | ((num >>> 7) & 127));
            this.u8(128 | ((num >>> 14) & 127));
            this.u8(num >>> 21);
        }
        else {
            var lo32 = num | 0;
            if (lo32 < 0)
                lo32 += 4294967296;
            var hi32 = (num - lo32) / 4294967296;
            if (num <= 34359738367) {
                this.u8(128 | (num & 127));
                this.u8(128 | ((num >>> 7) & 127));
                this.u8(128 | ((num >>> 14) & 127));
                this.u8(128 | ((num >>> 21) & 127));
                this.u8((hi32 << 4) | (num >>> 28));
            }
            else if (num <= 4398046511103) {
                this.u8(128 | (num & 127));
                this.u8(128 | ((num >>> 7) & 127));
                this.u8(128 | ((num >>> 14) & 127));
                this.u8(128 | ((num >>> 21) & 127));
                this.u8(128 | ((hi32 & 7) << 4) | (num >>> 28));
                this.u8(hi32 >>> 3);
            }
            else if (num <= 562949953421311) {
                this.u8(128 | (num & 127));
                this.u8(128 | ((num >>> 7) & 127));
                this.u8(128 | ((num >>> 14) & 127));
                this.u8(128 | ((num >>> 21) & 127));
                this.u8(128 | ((hi32 & 7) << 4) | (num >>> 28));
                this.u8(128 | ((hi32 & 1016) >>> 3));
                this.u8(hi32 >>> 10);
            }
            else {
                this.u8(128 | (num & 127));
                this.u8(128 | ((num >>> 7) & 127));
                this.u8(128 | ((num >>> 14) & 127));
                this.u8(128 | ((num >>> 21) & 127));
                this.u8(128 | ((hi32 & 7) << 4) | (num >>> 28));
                this.u8(128 | ((hi32 & 1016) >>> 3));
                this.u8(128 | ((hi32 & 130048) >>> 10));
                this.u8(hi32 >>> 17);
            }
        }
    };
    AbstractEncoder.prototype.vuint39 = function (num) {
        if (num <= 127) {
            this.u8(num);
        }
        else if (num <= 16383) {
            this.u8(128 | (num & 127));
            this.u8(num >>> 7);
        }
        else if (num <= 2097151) {
            this.u8(128 | (num & 127));
            this.u8(128 | ((num >>> 7) & 127));
            this.u8(num >>> 14);
        }
        else if (num <= 268435455) {
            this.u8(128 | (num & 127));
            this.u8(128 | ((num >>> 7) & 127));
            this.u8(128 | ((num >>> 14) & 127));
            this.u8(num >>> 21);
        }
        else {
            var lo32 = num | 0;
            if (lo32 < 0)
                lo32 += 4294967296;
            var hi32 = (num - lo32) / 4294967296;
            if (num <= 34359738367) {
                this.u8(128 | (num & 127));
                this.u8(128 | ((num >>> 7) & 127));
                this.u8(128 | ((num >>> 14) & 127));
                this.u8(128 | ((num >>> 21) & 127));
                this.u8((hi32 << 4) | (num >>> 28));
            }
            else if (num <= 4398046511103) {
                this.u8(128 | (num & 127));
                this.u8(128 | ((num >>> 7) & 127));
                this.u8(128 | ((num >>> 14) & 127));
                this.u8(128 | ((num >>> 21) & 127));
                this.u8(128 | ((hi32 & 7) << 4) | (num >>> 28));
                this.u8((hi32 >>> 3) & 15);
            }
        }
    };
    AbstractEncoder.prototype.b1vuint56 = function (flag, num) {
        if (num <= 63) {
            this.u8((flag ? 128 : 0) | num);
        }
        else {
            var firstByteMask = flag ? 192 : 64;
            if (num <= 8191) {
                this.u8(firstByteMask | (num & 63));
                this.u8(num >>> 6);
            }
            else if (num <= 1048575) {
                this.u8(firstByteMask | (num & 63));
                this.u8(128 | ((num >>> 6) & 127));
                this.u8(num >>> 13);
            }
            else if (num <= 134217727) {
                this.u8(firstByteMask | (num & 63));
                this.u8(128 | ((num >>> 6) & 127));
                this.u8(128 | ((num >>> 13) & 127));
                this.u8(num >>> 20);
            }
            else {
                var lo32 = num | 0;
                if (lo32 < 0)
                    lo32 += 4294967296;
                var hi32 = (num - lo32) / 4294967296;
                if (num <= 17179869183) {
                    this.u8(firstByteMask | (num & 63));
                    this.u8(128 | ((num >>> 6) & 127));
                    this.u8(128 | ((num >>> 13) & 127));
                    this.u8(128 | ((num >>> 20) & 127));
                    this.u8((hi32 << 5) | (num >>> 27));
                }
                else if (num <= 2199023255551) {
                    this.u8(firstByteMask | (num & 63));
                    this.u8(128 | ((num >>> 6) & 127));
                    this.u8(128 | ((num >>> 13) & 127));
                    this.u8(128 | ((num >>> 20) & 127));
                    this.u8(128 | ((hi32 & 3) << 5) | (num >>> 27));
                    this.u8(hi32 >>> 2);
                }
                else if (num <= 281474976710655) {
                    this.u8(firstByteMask | (num & 63));
                    this.u8(128 | ((num >>> 6) & 127));
                    this.u8(128 | ((num >>> 13) & 127));
                    this.u8(128 | ((num >>> 20) & 127));
                    this.u8(128 | ((hi32 & 3) << 5) | (num >>> 27));
                    this.u8(128 | ((hi32 & 508) >>> 2));
                    this.u8(hi32 >>> 9);
                }
                else {
                    this.u8(firstByteMask | (num & 63));
                    this.u8(128 | ((num >>> 6) & 127));
                    this.u8(128 | ((num >>> 13) & 127));
                    this.u8(128 | ((num >>> 20) & 127));
                    this.u8(128 | ((hi32 & 3) << 5) | (num >>> 27));
                    this.u8(128 | ((hi32 & 508) >>> 2));
                    this.u8(128 | ((hi32 & 65024) >>> 9));
                    this.u8(hi32 >>> 16);
                }
            }
        }
    };
    AbstractEncoder.prototype.b1vuint28 = function (flag, num) {
        if (num <= 63) {
            this.u8((flag ? 128 : 0) | num);
        }
        else {
            var firstByteMask = flag ? 192 : 64;
            if (num <= 8191) {
                this.u8(firstByteMask | (num & 63));
                this.u8(num >>> 6);
            }
            else if (num <= 1048575) {
                this.u8(firstByteMask | (num & 63));
                this.u8(128 | ((num >>> 6) & 127));
                this.u8(num >>> 13);
            }
            else {
                this.u8(firstByteMask | (num & 63));
                this.u8(128 | ((num >>> 6) & 127));
                this.u8(128 | ((num >>> 13) & 127));
                this.u8(num >>> 20);
            }
        }
    };
    AbstractEncoder.prototype.uint53vuint39 = function (x, z) {
        var x1 = x | 0;
        if (x1 < 0)
            x1 += 4294967296;
        var x2 = (x - x1) / 4294967296;
        this.u32(x1);
        this.u16(x2 & 0xffff);
        var fiveXBits = (x2 >>> 16) << 3;
        var twoZBits = (z >>> 8) & 3;
        var zFitsIn10Bits = z <= 1023;
        this.u8(fiveXBits | (zFitsIn10Bits ? 0 : 4) | twoZBits);
        this.u8(z & 0xff);
        if (zFitsIn10Bits)
            return;
        if (z <= 131071) {
            this.u8(z >>> 10);
        }
        else if (z <= 16777215) {
            this.u8(128 | ((z >>> 10) & 127));
            this.u8(z >>> 17);
        }
        else if (z <= 2147483647) {
            this.u8(128 | ((z >>> 10) & 127));
            this.u8(128 | ((z >>> 17) & 127));
            this.u8(z >>> 24);
        }
        else {
            var z1 = z | 0;
            if (z1 < 0)
                z1 += 4294967296;
            var z2 = (z - z1) / 4294967296;
            this.u8(128 | ((z1 >>> 10) & 127));
            this.u8(128 | ((z1 >>> 17) & 127));
            this.u8(128 | ((z1 >>> 24) & 127));
            this.u8((z1 >>> 31) | (z2 << 1));
        }
    };
    return AbstractEncoder;
}(Encoder_1.Encoder));
exports.AbstractEncoder = AbstractEncoder;
