"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServerDecoder = void 0;
var clock_1 = require("../../../json-crdt-patch/clock");
var model_1 = require("../../model");
var DocRootType_1 = require("../../types/lww-doc-root/DocRootType");
var constants_1 = require("../../../json-crdt-patch/constants");
var constants_2 = require("../../constants");
var ConstantType_1 = require("../../types/const/ConstantType");
var ObjectChunk_1 = require("../../types/lww-object/ObjectChunk");
var ObjectType_1 = require("../../types/lww-object/ObjectType");
var ValueType_1 = require("../../types/lww-value/ValueType");
var ArrayChunk_1 = require("../../types/rga-array/ArrayChunk");
var ArrayType_1 = require("../../types/rga-array/ArrayType");
var StringChunk_1 = require("../../types/rga-string/StringChunk");
var StringType_1 = require("../../types/rga-string/StringType");
var ServerDecoder = (function () {
    function ServerDecoder() {
    }
    ServerDecoder.prototype.decode = function (data) {
        this.time = data[0];
        var doc = model_1.Model.withServerClock(this.time);
        var rootId = this.ts(data, 1);
        var rootNode = data[2] ? this.decodeNode(doc, data[2]) : null;
        doc.root = new DocRootType_1.DocRootType(doc, rootId, rootNode);
        return doc;
    };
    ServerDecoder.prototype.ts = function (arr, index) {
        var timeDiff = arr[index];
        return new clock_1.ServerTimestamp(this.time - timeDiff);
    };
    ServerDecoder.prototype.decodeNode = function (doc, data) {
        switch (data) {
            case 1:
                return constants_2.NULL;
            case 2:
                return constants_2.FALSE;
            case 3:
                return constants_2.TRUE;
            case 4:
                return constants_2.UNDEFINED;
        }
        if (data instanceof Array) {
            switch (data[0]) {
                case 0:
                    return this.decodeObj(doc, data);
                case 1:
                    return this.decodeArr(doc, data);
                case 2:
                    return this.decodeStr(doc, data);
                case 3:
                    return this.decodeVal(doc, data);
                case 4:
                    return this.decodeConst(doc, data);
            }
        }
        throw new Error('UNKNOWN_NODE');
    };
    ServerDecoder.prototype.decodeObj = function (doc, data) {
        var id = this.ts(data, 1);
        var obj = new ObjectType_1.ObjectType(doc, id);
        var length = data.length;
        for (var i = 2; i < length; i += 3) {
            var key = data[i];
            var keyId = this.ts(data, i + 1);
            var node = this.decodeNode(doc, data[i + 2]);
            var chunk = new ObjectChunk_1.ObjectChunk(keyId, node);
            obj.putChunk(key, chunk);
        }
        doc.nodes.index(obj);
        return obj;
    };
    ServerDecoder.prototype.decodeArr = function (doc, data) {
        var _this = this;
        var id = this.ts(data, 1);
        var obj = new ArrayType_1.ArrayType(doc, id);
        var length = data.length;
        for (var i = 2; i < length; i += 2) {
            var chunkId = this.ts(data, i);
            var content = data[i + 1];
            var chunk = void 0;
            if (typeof content === 'number') {
                chunk = new ArrayChunk_1.ArrayChunk(chunkId, undefined);
                chunk.deleted = content;
            }
            else {
                var nodes = content.map(function (c) { return _this.decodeNode(doc, c); });
                chunk = new ArrayChunk_1.ArrayChunk(chunkId, nodes);
            }
            obj.append(chunk);
        }
        doc.nodes.index(obj);
        return obj;
    };
    ServerDecoder.prototype.decodeStr = function (doc, data) {
        var id = this.ts(data, 1);
        var obj = new StringType_1.StringType(doc, id);
        var length = data.length;
        for (var i = 2; i < length; i += 2) {
            var chunkId = this.ts(data, i);
            var content = data[i + 1];
            var chunk = void 0;
            if (typeof content === 'number') {
                chunk = new StringChunk_1.StringChunk(chunkId, undefined);
                chunk.deleted = content;
            }
            else {
                chunk = new StringChunk_1.StringChunk(chunkId, content);
            }
            obj.append(chunk);
        }
        doc.nodes.index(obj);
        return obj;
    };
    ServerDecoder.prototype.decodeVal = function (doc, data) {
        var id = this.ts(data, 1);
        var writeId = this.ts(data, 2);
        var value = data[3];
        var obj = new ValueType_1.ValueType(id, writeId, value);
        doc.nodes.index(obj);
        return obj;
    };
    ServerDecoder.prototype.decodeConst = function (doc, data) {
        var value = data[1];
        var obj = new ConstantType_1.ConstantType(constants_1.ORIGIN, value);
        return obj;
    };
    return ServerDecoder;
}());
exports.ServerDecoder = ServerDecoder;
