"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractDecoder = void 0;
var tslib_1 = require("tslib");
var constants_1 = require("../../../json-crdt-patch/constants");
var constants_2 = require("../../constants");
var ConstantType_1 = require("../../types/const/ConstantType");
var DocRootType_1 = require("../../types/lww-doc-root/DocRootType");
var ObjectChunk_1 = require("../../types/lww-object/ObjectChunk");
var ObjectType_1 = require("../../types/lww-object/ObjectType");
var ValueType_1 = require("../../types/lww-value/ValueType");
var ArrayChunk_1 = require("../../types/rga-array/ArrayChunk");
var ArrayType_1 = require("../../types/rga-array/ArrayType");
var StringChunk_1 = require("../../types/rga-string/StringChunk");
var StringType_1 = require("../../types/rga-string/StringType");
var AbstractDecoder = (function () {
    function AbstractDecoder() {
    }
    AbstractDecoder.prototype.decodeRoot = function (doc, _a) {
        var id = _a.id, node = _a.node;
        var ts = this.decodeTimestamp(id);
        var jsonNode = node ? this.decodeNode(doc, node) : null;
        var root = new DocRootType_1.DocRootType(doc, ts, jsonNode);
        doc.root = root;
    };
    AbstractDecoder.prototype.decodeNode = function (doc, node) {
        switch (node.type) {
            case 'obj':
                return this.decodeObj(doc, node);
            case 'arr':
                return this.decodeArr(doc, node);
            case 'str':
                return this.decodeStr(doc, node);
            case 'val':
                return this.decodeVal(doc, node);
            case 'const':
                return this.decodeConst(doc, node);
        }
        throw new Error('UNKNOWN_NODE');
    };
    AbstractDecoder.prototype.decodeObj = function (doc, node) {
        var e_1, _a;
        var id = this.decodeTimestamp(node.id);
        var obj = new ObjectType_1.ObjectType(doc, id);
        var keys = Object.keys(node.chunks);
        try {
            for (var keys_1 = tslib_1.__values(keys), keys_1_1 = keys_1.next(); !keys_1_1.done; keys_1_1 = keys_1.next()) {
                var key = keys_1_1.value;
                var val = node.chunks[key];
                obj.putChunk(key, new ObjectChunk_1.ObjectChunk(this.decodeTimestamp(val.id), this.decodeNode(doc, val.node)));
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (keys_1_1 && !keys_1_1.done && (_a = keys_1.return)) _a.call(keys_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        doc.nodes.index(obj);
        return obj;
    };
    AbstractDecoder.prototype.decodeArr = function (doc, node) {
        var e_2, _a;
        var obj = new ArrayType_1.ArrayType(doc, this.decodeTimestamp(node.id));
        try {
            for (var _b = tslib_1.__values(node.chunks), _c = _b.next(); !_c.done; _c = _b.next()) {
                var c = _c.value;
                obj.append(this.decodeArrChunk(doc, c));
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
        doc.nodes.index(obj);
        return obj;
    };
    AbstractDecoder.prototype.decodeArrChunk = function (doc, c) {
        var _this = this;
        var id = this.decodeTimestamp(c.id);
        if (typeof c.span === 'number') {
            var chunk = new ArrayChunk_1.ArrayChunk(id, undefined);
            chunk.deleted = c.span;
            return chunk;
        }
        else
            return new ArrayChunk_1.ArrayChunk(id, c.nodes.map(function (n) { return _this.decodeNode(doc, n); }));
    };
    AbstractDecoder.prototype.decodeStr = function (doc, node) {
        var e_3, _a;
        var obj = new StringType_1.StringType(doc, this.decodeTimestamp(node.id));
        try {
            for (var _b = tslib_1.__values(node.chunks), _c = _b.next(); !_c.done; _c = _b.next()) {
                var c = _c.value;
                obj.append(this.decodeStrChunk(doc, c));
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_3) throw e_3.error; }
        }
        doc.nodes.index(obj);
        return obj;
    };
    AbstractDecoder.prototype.decodeStrChunk = function (doc, c) {
        var id = this.decodeTimestamp(c.id);
        if (typeof c.span === 'number') {
            var chunk = new StringChunk_1.StringChunk(id, undefined);
            chunk.deleted = c.span;
            return chunk;
        }
        else
            return new StringChunk_1.StringChunk(id, c.value);
    };
    AbstractDecoder.prototype.decodeVal = function (doc, node) {
        var obj = new ValueType_1.ValueType(this.decodeTimestamp(node.id), this.decodeTimestamp(node.writeId), node.value);
        doc.nodes.index(obj);
        return obj;
    };
    AbstractDecoder.prototype.decodeConst = function (doc, node) {
        switch (node.value) {
            case null:
                return constants_2.NULL;
            case true:
                return constants_2.TRUE;
            case false:
                return constants_2.FALSE;
            case undefined:
                return constants_2.UNDEFINED;
        }
        return new ConstantType_1.ConstantType(constants_1.ORIGIN, node.value);
    };
    return AbstractDecoder;
}());
exports.AbstractDecoder = AbstractDecoder;
