"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractEncoder = void 0;
var tslib_1 = require("tslib");
var ConstantType_1 = require("../../types/const/ConstantType");
var ObjectType_1 = require("../../types/lww-object/ObjectType");
var ValueType_1 = require("../../types/lww-value/ValueType");
var ArrayType_1 = require("../../types/rga-array/ArrayType");
var StringType_1 = require("../../types/rga-string/StringType");
var AbstractEncoder = (function () {
    function AbstractEncoder() {
    }
    AbstractEncoder.prototype.encodeRoot = function (root) {
        return {
            type: 'root',
            id: this.encodeTimestamp(root.id),
            node: root.node ? this.encodeNode(root.node) : null,
        };
    };
    AbstractEncoder.prototype.encodeNode = function (node) {
        if (node instanceof ObjectType_1.ObjectType)
            return this.encodeObj(node);
        else if (node instanceof ArrayType_1.ArrayType)
            return this.encodeArr(node);
        else if (node instanceof StringType_1.StringType)
            return this.encodeStr(node);
        else if (node instanceof ValueType_1.ValueType)
            return this.encodeVal(node);
        else if (node instanceof ConstantType_1.ConstantType)
            return this.encodeConst(node);
        throw new Error('UNKNOWN_NODE');
    };
    AbstractEncoder.prototype.encodeObj = function (obj) {
        var e_1, _a;
        var chunks = {};
        try {
            for (var _b = tslib_1.__values(obj.latest.entries()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = tslib_1.__read(_c.value, 2), key = _d[0], entry = _d[1];
                chunks[key] = {
                    id: this.encodeTimestamp(entry.id),
                    node: this.encodeNode(entry.node),
                };
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return {
            type: 'obj',
            id: this.encodeTimestamp(obj.id),
            chunks: chunks,
        };
    };
    AbstractEncoder.prototype.encodeArr = function (obj) {
        var e_2, _a;
        var chunks = [];
        try {
            for (var _b = tslib_1.__values(obj.chunks()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var chunk = _c.value;
                chunks.push(this.encodeArrChunk(chunk));
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return {
            type: 'arr',
            id: this.encodeTimestamp(obj.id),
            chunks: chunks,
        };
    };
    AbstractEncoder.prototype.encodeArrChunk = function (chunk) {
        var _this = this;
        if (chunk.deleted) {
            var tombstone = {
                id: this.encodeTimestamp(chunk.id),
                span: chunk.deleted,
            };
            return tombstone;
        }
        var res = {
            id: this.encodeTimestamp(chunk.id),
            nodes: chunk.nodes.map(function (n) { return _this.encodeNode(n); }),
        };
        return res;
    };
    AbstractEncoder.prototype.encodeStr = function (obj) {
        var e_3, _a;
        var chunks = [];
        try {
            for (var _b = tslib_1.__values(obj.chunks()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var chunk = _c.value;
                chunks.push(this.encodeStrChunk(chunk));
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_3) throw e_3.error; }
        }
        return {
            type: 'str',
            id: this.encodeTimestamp(obj.id),
            chunks: chunks,
        };
    };
    AbstractEncoder.prototype.encodeStrChunk = function (chunk) {
        if (chunk.deleted) {
            var tombstone = {
                id: this.encodeTimestamp(chunk.id),
                span: chunk.deleted,
            };
            return tombstone;
        }
        var res = {
            id: this.encodeTimestamp(chunk.id),
            value: chunk.str,
        };
        return res;
    };
    AbstractEncoder.prototype.encodeVal = function (obj) {
        return {
            type: 'val',
            id: this.encodeTimestamp(obj.id),
            writeId: this.encodeTimestamp(obj.writeId),
            value: obj.value,
        };
    };
    AbstractEncoder.prototype.encodeConst = function (obj) {
        return {
            type: 'const',
            value: obj.value,
        };
    };
    return AbstractEncoder;
}());
exports.AbstractEncoder = AbstractEncoder;
