"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonPatchDraft = void 0;
var tslib_1 = require("tslib");
var Draft_1 = require("../../json-crdt-patch/Draft");
var op_1 = require("../../json-patch/op");
var ObjectType_1 = require("../types/lww-object/ObjectType");
var ArrayType_1 = require("../types/rga-array/ArrayType");
var constants_1 = require("../../json-crdt-patch/constants");
var json_pointer_1 = require("../../json-pointer");
var isEqual = require('fast-deep-equal');
var JsonPatchDraft = (function (_super) {
    tslib_1.__extends(JsonPatchDraft, _super);
    function JsonPatchDraft(model) {
        var _this = _super.call(this) || this;
        _this.model = model;
        return _this;
    }
    JsonPatchDraft.prototype.applyOps = function (ops) {
        var e_1, _a;
        try {
            for (var ops_1 = tslib_1.__values(ops), ops_1_1 = ops_1.next(); !ops_1_1.done; ops_1_1 = ops_1.next()) {
                var op = ops_1_1.value;
                this.applyOp(op);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (ops_1_1 && !ops_1_1.done && (_a = ops_1.return)) _a.call(ops_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    JsonPatchDraft.prototype.applyOp = function (op) {
        if (op instanceof op_1.OpAdd)
            this.applyOpAdd(op);
        else if (op instanceof op_1.OpRemove)
            this.applyOpRemove(op);
        else if (op instanceof op_1.OpReplace)
            this.applyOpReplace(op);
        else if (op instanceof op_1.OpMove)
            this.applyOpMove(op);
        else if (op instanceof op_1.OpCopy)
            this.applyOpCopy(op);
        else if (op instanceof op_1.OpTest)
            this.applyOpTest(op);
    };
    JsonPatchDraft.prototype.applyOpAdd = function (op) {
        var builder = this.builder;
        var steps = op.path;
        if (!steps.length)
            this.setRoot(op.value);
        else {
            var objSteps = steps.slice(0, steps.length - 1);
            var node = this.model.api.find(objSteps);
            var key = steps[steps.length - 1];
            if (node instanceof ObjectType_1.ObjectType) {
                builder.setKeys(node.id, [[String(key), builder.json(op.value)]]);
            }
            else if (node instanceof ArrayType_1.ArrayType) {
                var value = builder.json(op.value);
                if (key === '-') {
                    var length_1 = node.size();
                    var after = node.findId(length_1);
                    builder.insArr(node.id, after, [value]);
                }
                else {
                    var index = ~~key;
                    if ('' + index !== key)
                        throw new Error('INVALID_INDEX');
                    if (!index)
                        builder.insArr(node.id, node.id, [value]);
                    else {
                        var after = node.findId(index - 1);
                        builder.insArr(node.id, after, [value]);
                    }
                }
            }
            else
                throw new Error('NOT_FOUND');
        }
    };
    JsonPatchDraft.prototype.applyOpRemove = function (op) {
        var builder = this.builder;
        var steps = op.path;
        if (!steps.length)
            this.setRoot(null);
        else {
            var objSteps = steps.slice(0, steps.length - 1);
            var node = this.model.api.find(objSteps);
            var key = steps[steps.length - 1];
            if (node instanceof ObjectType_1.ObjectType) {
                var stringKey = String(key);
                if (node.get(stringKey) === undefined)
                    throw new Error('NOT_FOUND');
                builder.setKeys(node.id, [[stringKey, constants_1.UNDEFINED_ID]]);
            }
            else if (node instanceof ArrayType_1.ArrayType) {
                var key_1 = steps[steps.length - 1];
                var index = ~~key_1;
                if ('' + index !== key_1)
                    throw new Error('INVALID_INDEX');
                var id = node.findId(index);
                builder.del(node.id, id, 1);
            }
            else
                throw new Error('NOT_FOUND');
        }
    };
    JsonPatchDraft.prototype.applyOpReplace = function (op) {
        var path = op.path, value = op.value;
        this.applyOpRemove(new op_1.OpRemove(path, undefined));
        this.applyOpAdd(new op_1.OpAdd(path, value));
    };
    JsonPatchDraft.prototype.applyOpMove = function (op) {
        var path = op.path, from = op.from;
        if (json_pointer_1.isChild(from, path))
            throw new Error('INVALID_CHILD');
        var json = this.json(from);
        this.applyOpRemove(new op_1.OpRemove(from, undefined));
        this.applyOpAdd(new op_1.OpAdd(path, json));
    };
    JsonPatchDraft.prototype.applyOpCopy = function (op) {
        var path = op.path, from = op.from;
        var json = this.json(from);
        this.applyOpAdd(new op_1.OpAdd(path, json));
    };
    JsonPatchDraft.prototype.applyOpTest = function (op) {
        var path = op.path, value = op.value;
        var json = this.json(path);
        if (!isEqual(json, value))
            throw new Error('TEST');
    };
    JsonPatchDraft.prototype.get = function (steps) {
        var _a, _b;
        if (!steps.length)
            return this.model.toJson();
        else {
            var objSteps = steps.slice(0, steps.length - 1);
            var node = this.model.api.find(objSteps);
            var key = steps[steps.length - 1];
            if (node instanceof ObjectType_1.ObjectType) {
                return (_a = node.getNode(String(key))) === null || _a === void 0 ? void 0 : _a.toJson();
            }
            else if (node instanceof ArrayType_1.ArrayType) {
                var index = ~~key;
                if ('' + index !== key)
                    throw new Error('INVALID_INDEX');
                var nodeId = node.findValue(index);
                return (_b = this.model.node(nodeId)) === null || _b === void 0 ? void 0 : _b.toJson();
            }
        }
        return undefined;
    };
    JsonPatchDraft.prototype.json = function (steps) {
        var json = this.get(steps);
        if (json === undefined)
            throw new Error('NOT_FOUND');
        return json;
    };
    JsonPatchDraft.prototype.setRoot = function (json) {
        var builder = this.builder;
        builder.root(builder.json(json));
    };
    return JsonPatchDraft;
}(Draft_1.Draft));
exports.JsonPatchDraft = JsonPatchDraft;
