"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StringType = void 0;
var tslib_1 = require("tslib");
var StringChunk_1 = require("./StringChunk");
var StringOriginChunk_1 = require("./StringOriginChunk");
var StringType = (function () {
    function StringType(doc, id) {
        this.doc = doc;
        this.id = id;
        this.start = this.end = new StringOriginChunk_1.StringOriginChunk(id);
    }
    StringType.prototype.onInsert = function (op) {
        var curr = this.end;
        while (curr) {
            if (curr.id.overlap(curr.span(), op.id, op.span()))
                return;
            if (curr.id.inSpan(curr.span(), op.after, 1))
                break;
            curr = curr.left;
        }
        if (!curr)
            return;
        var isOriginChunk = curr instanceof StringOriginChunk_1.StringOriginChunk;
        if (!curr.deleted && !isOriginChunk) {
            var doesAfterMatch = curr.id.getSessionId() === op.after.getSessionId() && curr.id.time + curr.span() - 1 === op.after.time;
            var isIdSameSession = curr.id.getSessionId() === op.id.getSessionId();
            var isIdIncreasingWithoutAGap = curr.id.time + curr.span() === op.id.time;
            if (doesAfterMatch && isIdSameSession && isIdIncreasingWithoutAGap) {
                curr.merge(op.substring);
                return;
            }
        }
        var chunk = new StringChunk_1.StringChunk(op.id, op.substring);
        var targetsLastElementInChunk = op.after.time === curr.id.time + curr.span() - 1;
        if (targetsLastElementInChunk) {
            while (curr.right && curr.right.id.compare(op.id) > 0)
                curr = curr.right;
            this.insertChunk(chunk, curr);
            return;
        }
        this.splitChunk(curr, op.after.time);
        this.insertChunk(chunk, curr);
    };
    StringType.prototype.onDelete = function (op) {
        var e_1, _a;
        var after = op.after, length = op.length;
        var chunk = this.end;
        var chunks = [];
        while (chunk) {
            if (chunk.id.overlap(chunk.span(), after, length))
                chunks.push(chunk);
            if (chunk.id.inSpan(chunk.span(), after, 1))
                break;
            chunk = chunk.left;
        }
        try {
            for (var chunks_1 = tslib_1.__values(chunks), chunks_1_1 = chunks_1.next(); !chunks_1_1.done; chunks_1_1 = chunks_1.next()) {
                var c = chunks_1_1.value;
                this.deleteInChunk(c, after.time, after.time + length - 1);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (chunks_1_1 && !chunks_1_1.done && (_a = chunks_1.return)) _a.call(chunks_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    StringType.prototype.insertChunk = function (chunk, after) {
        chunk.left = after;
        chunk.right = after.right;
        if (after.right)
            after.right.left = chunk;
        else
            this.end = chunk;
        after.right = chunk;
    };
    StringType.prototype.splitChunk = function (chunk, time) {
        var newChunk = chunk.split(time);
        this.insertChunk(newChunk, chunk);
        return newChunk;
    };
    StringType.prototype.deleteInChunk = function (chunk, t1, t2) {
        var c1 = chunk.id.time;
        var c2 = c1 + chunk.span() - 1;
        if (t1 <= c1) {
            if (t2 >= c2)
                chunk.delete();
            else {
                this.splitChunk(chunk, t2);
                chunk.delete();
            }
        }
        else {
            if (t2 >= c2)
                this.splitChunk(chunk, t1 - 1).delete();
            else {
                this.splitChunk(chunk, t2);
                this.splitChunk(chunk, t1 - 1).delete();
            }
        }
    };
    StringType.prototype.findId = function (index) {
        var chunk = this.start;
        var cnt = 0;
        var next = index + 1;
        while (chunk) {
            if (chunk.str) {
                cnt += chunk.str.length;
                if (cnt >= next)
                    return chunk.id.tick(chunk.str.length - (cnt - index));
            }
            chunk = chunk.right;
        }
        throw new Error('OUT_OF_BOUNDS');
    };
    StringType.prototype.findIdSpan = function (index, length) {
        var chunk = this.start;
        var cnt = 0;
        var next = index + 1;
        while (chunk) {
            if (chunk.str) {
                cnt += chunk.str.length;
                if (cnt >= next) {
                    var remaining = cnt - index;
                    if (remaining >= length)
                        return [chunk.id.interval(chunk.span() - remaining, length)];
                    length -= remaining;
                    var result = [chunk.id.interval(chunk.span() - remaining, remaining)];
                    while (chunk.right) {
                        chunk = chunk.right;
                        if (chunk.deleted)
                            continue;
                        var span = chunk.span();
                        if (span >= length) {
                            result.push(chunk.id.interval(0, length));
                            break;
                        }
                        result.push(chunk.id.interval(0, span));
                        length -= span;
                    }
                    return result;
                }
            }
            chunk = chunk.right;
        }
        throw new Error('OUT_OF_BOUNDS');
    };
    StringType.prototype.append = function (chunk) {
        var last = this.end;
        last.right = chunk;
        chunk.left = last;
        this.end = chunk;
    };
    StringType.prototype.toJson = function () {
        var str = '';
        var curr = this.start;
        while ((curr = curr.right))
            if (curr.str)
                str += curr.str;
        return str;
    };
    StringType.prototype.clone = function (doc) {
        var copy = new StringType(this.doc, this.id);
        var i = this.start;
        var j = copy.start;
        while (i.right) {
            var cloned = i.right.clone();
            j.right = cloned;
            cloned.left = j;
            j = cloned;
            i = i.right;
        }
        copy.end = j;
        doc.nodes.index(copy);
        return copy;
    };
    StringType.prototype.children = function () { return tslib_1.__generator(this, function (_a) {
        return [2];
    }); };
    StringType.prototype.chunks = function () {
        var curr;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    curr = this.start;
                    _a.label = 1;
                case 1:
                    if (!(curr = curr.right)) return [3, 3];
                    return [4, curr];
                case 2:
                    _a.sent();
                    return [3, 1];
                case 3: return [2];
            }
        });
    };
    StringType.prototype.size = function () {
        var curr = this.start;
        var size = 0;
        while ((curr = curr.right))
            size++;
        return size;
    };
    StringType.prototype.length = function () {
        var curr = this.start;
        var size = 0;
        while ((curr = curr.right))
            if (curr.str)
                size += curr.str.length;
        return size;
    };
    StringType.prototype.toString = function (tab) {
        if (tab === void 0) { tab = ''; }
        var str = tab + "StringType(" + this.id.toDisplayString() + ")";
        var curr = this.start;
        while (curr) {
            str += "\n" + curr.toString(tab + '  ');
            curr = curr.right;
        }
        return str;
    };
    return StringType;
}());
exports.StringType = StringType;
