"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CachedKeyDecoder = exports.utf8DecodeJs = void 0;
var tslib_1 = require("tslib");
function utf8DecodeJs(bytes, inputOffset, byteLength) {
    var offset = inputOffset;
    var end = offset + byteLength;
    var units = [];
    var result = '';
    while (offset < end) {
        var byte1 = bytes[offset++];
        if ((byte1 & 0x80) === 0) {
            units.push(byte1);
        }
        else if ((byte1 & 0xe0) === 0xc0) {
            var byte2 = bytes[offset++] & 0x3f;
            units.push(((byte1 & 0x1f) << 6) | byte2);
        }
        else if ((byte1 & 0xf0) === 0xe0) {
            var byte2 = bytes[offset++] & 0x3f;
            var byte3 = bytes[offset++] & 0x3f;
            units.push(((byte1 & 0x1f) << 12) | (byte2 << 6) | byte3);
        }
        else if ((byte1 & 0xf8) === 0xf0) {
            var byte2 = bytes[offset++] & 0x3f;
            var byte3 = bytes[offset++] & 0x3f;
            var byte4 = bytes[offset++] & 0x3f;
            var unit = ((byte1 & 0x07) << 0x12) | (byte2 << 0x0c) | (byte3 << 0x06) | byte4;
            if (unit > 0xffff) {
                unit -= 0x10000;
                units.push(((unit >>> 10) & 0x3ff) | 0xd800);
                unit = 0xdc00 | (unit & 0x3ff);
            }
            units.push(unit);
        }
        else {
            units.push(byte1);
        }
        if (units.length >= 1000) {
            result += String.fromCharCode.apply(String, tslib_1.__spreadArray([], tslib_1.__read(units)));
            units.length = 0;
        }
    }
    if (units.length > 0)
        result += String.fromCharCode.apply(String, tslib_1.__spreadArray([], tslib_1.__read(units)));
    return result;
}
exports.utf8DecodeJs = utf8DecodeJs;
var CachedKeyDecoder = (function () {
    function CachedKeyDecoder() {
        this.caches = [];
        for (var i = 0; i < 31; i++)
            this.caches.push([]);
    }
    CachedKeyDecoder.prototype.get = function (bytes, inputOffset, byteLength) {
        var records = this.caches[byteLength - 1];
        var len = records.length;
        FIND_CHUNK: for (var i = 0; i < len; i++) {
            var record = records[i];
            var recordBytes = record.bytes;
            for (var j = 0; j < byteLength; j++) {
                if (recordBytes[j] !== bytes[inputOffset + j]) {
                    continue FIND_CHUNK;
                }
            }
            return record.value;
        }
        return null;
    };
    CachedKeyDecoder.prototype.store = function (bytes, value) {
        var records = this.caches[bytes.length - 1];
        var record = { bytes: bytes, value: value };
        if (records.length >= 16) {
            records[(Math.random() * records.length) | 0] = record;
        }
        else {
            records.push(record);
        }
    };
    CachedKeyDecoder.prototype.decode = function (bytes, offset, size) {
        if (!size)
            return '';
        var cachedValue = this.get(bytes, offset, size);
        if (cachedValue != null)
            return cachedValue;
        var value = utf8DecodeJs(bytes, offset, size);
        var slicedCopyOfBytes = Uint8Array.prototype.slice.call(bytes, offset, offset + size);
        this.store(slicedCopyOfBytes, value);
        return value;
    };
    return CachedKeyDecoder;
}());
exports.CachedKeyDecoder = CachedKeyDecoder;
