"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decode = exports.compactToPredicateOp = exports.compactToOp = void 0;
var tslib_1 = require("tslib");
var OpAdd_1 = require("../../op/OpAdd");
var OpRemove_1 = require("../../op/OpRemove");
var OpReplace_1 = require("../../op/OpReplace");
var OpMove_1 = require("../../op/OpMove");
var OpCopy_1 = require("../../op/OpCopy");
var OpTest_1 = require("../../op/OpTest");
var OpFlip_1 = require("../../op/OpFlip");
var OpInc_1 = require("../../op/OpInc");
var OpStrIns_1 = require("../../op/OpStrIns");
var OpStrDel_1 = require("../../op/OpStrDel");
var OpSplit_1 = require("../../op/OpSplit");
var OpMerge_1 = require("../../op/OpMerge");
var OpExtend_1 = require("../../op/OpExtend");
var OpDefined_1 = require("../../op/OpDefined");
var OpUndefined_1 = require("../../op/OpUndefined");
var OpTestType_1 = require("../../op/OpTestType");
var OpTestString_1 = require("../../op/OpTestString");
var OpTestStringLen_1 = require("../../op/OpTestStringLen");
var OpContains_1 = require("../../op/OpContains");
var OpEnds_1 = require("../../op/OpEnds");
var OpStarts_1 = require("../../op/OpStarts");
var OpIn_1 = require("../../op/OpIn");
var OpLess_1 = require("../../op/OpLess");
var OpMore_1 = require("../../op/OpMore");
var OpAnd_1 = require("../../op/OpAnd");
var OpOr_1 = require("../../op/OpOr");
var OpNot_1 = require("../../op/OpNot");
var OpMatches_1 = require("../../op/OpMatches");
var OpType_1 = require("../../op/OpType");
var json_pointer_1 = require("../../../json-pointer");
var compactToOp = function (op) {
    switch (op[0]) {
        case 0: return new OpAdd_1.OpAdd(json_pointer_1.toPath(op[1]), op[2]);
        case 1: return new OpRemove_1.OpRemove(json_pointer_1.toPath(op[1]), op[2]);
        case 2: return new OpReplace_1.OpReplace(json_pointer_1.toPath(op[1]), op[2], op[3]);
        case 4: return new OpMove_1.OpMove(json_pointer_1.toPath(op[1]), json_pointer_1.toPath(op[2]));
        case 3: return new OpCopy_1.OpCopy(json_pointer_1.toPath(op[1]), json_pointer_1.toPath(op[2]));
        case 8: return new OpFlip_1.OpFlip(json_pointer_1.toPath(op[1]));
        case 9: return new OpInc_1.OpInc(json_pointer_1.toPath(op[1]), op[2]);
        case 6: return new OpStrIns_1.OpStrIns(json_pointer_1.toPath(op[1]), op[2], op[3]);
        case 7: return new OpStrDel_1.OpStrDel(json_pointer_1.toPath(op[1]), op[2], op[3] || undefined, op[4]);
        case 10: return new OpSplit_1.OpSplit(json_pointer_1.toPath(op[1]), op[2], op[3] || null);
        case 11: return new OpMerge_1.OpMerge(json_pointer_1.toPath(op[1]), op[2], op[3] || null);
        case 12: return new OpExtend_1.OpExtend(json_pointer_1.toPath(op[1]), op[2], !!op[3]);
        default: return exports.compactToPredicateOp(op);
    }
};
exports.compactToOp = compactToOp;
var compactToPredicateOp = function (op) {
    switch (op[0]) {
        case 5: return new OpTest_1.OpTest(json_pointer_1.toPath(op[1]), op[2], !!op[3]);
        case 31: return new OpDefined_1.OpDefined(json_pointer_1.toPath(op[1]));
        case 38: return new OpUndefined_1.OpUndefined(json_pointer_1.toPath(op[1]));
        case 42: return new OpType_1.OpType(json_pointer_1.toPath(op[1]), op[2]);
        case 39: return new OpTestType_1.OpTestType(json_pointer_1.toPath(op[1]), op[2]);
        case 40: return new OpTestString_1.OpTestString(json_pointer_1.toPath(op[1]), op[2], op[3], !!op[4]);
        case 41: return new OpTestStringLen_1.OpTestStringLen(json_pointer_1.toPath(op[1]), op[2], !!op[3]);
        case 30: return new OpContains_1.OpContains(json_pointer_1.toPath(op[1]), op[2], !!op[3]);
        case 32: return new OpEnds_1.OpEnds(json_pointer_1.toPath(op[1]), op[2], !!op[3]);
        case 37: return new OpStarts_1.OpStarts(json_pointer_1.toPath(op[1]), op[2], !!op[3]);
        case 35: return new OpMatches_1.OpMatches(json_pointer_1.toPath(op[1]), op[2], !!op[3]);
        case 33: return new OpIn_1.OpIn(json_pointer_1.toPath(op[1]), op[2]);
        case 34: return new OpLess_1.OpLess(json_pointer_1.toPath(op[1]), op[2]);
        case 36: return new OpMore_1.OpMore(json_pointer_1.toPath(op[1]), op[2]);
        case 43: {
            var path_1 = json_pointer_1.toPath(op[1]);
            return new OpAnd_1.OpAnd(path_1, op[2].map(function (x) {
                var copy = tslib_1.__spreadArray([], tslib_1.__read(x));
                copy[1] = tslib_1.__spreadArray(tslib_1.__spreadArray([], tslib_1.__read(path_1)), tslib_1.__read(json_pointer_1.toPath(x[1])));
                return exports.compactToPredicateOp(copy);
            }));
        }
        case 45: {
            var path_2 = json_pointer_1.toPath(op[1]);
            return new OpOr_1.OpOr(path_2, op[2].map(function (x) {
                var copy = tslib_1.__spreadArray([], tslib_1.__read(x));
                copy[1] = tslib_1.__spreadArray(tslib_1.__spreadArray([], tslib_1.__read(path_2)), tslib_1.__read(json_pointer_1.toPath(x[1])));
                return exports.compactToPredicateOp(copy);
            }));
        }
        case 44: {
            var path_3 = json_pointer_1.toPath(op[1]);
            return new OpNot_1.OpNot(path_3, op[2].map(function (x) {
                var copy = tslib_1.__spreadArray([], tslib_1.__read(x));
                copy[1] = tslib_1.__spreadArray(tslib_1.__spreadArray([], tslib_1.__read(path_3)), tslib_1.__read(json_pointer_1.toPath(x[1])));
                return exports.compactToPredicateOp(copy);
            }));
        }
        default:
            throw new Error('OP_UNKNOWN');
    }
};
exports.compactToPredicateOp = compactToPredicateOp;
function decode(patch) {
    var ops = [];
    var length = patch.length;
    for (var i = 0; i < length; i++) {
        var op = exports.compactToOp(patch[i]);
        ops.push(op);
    }
    return ops;
}
exports.decode = decode;
