"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PersistentClient = void 0;
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var rpc_1 = require("../rpc");
var RpcApiCaller_1 = require("../rpc/RpcApiCaller");
var RpcDuplex_1 = require("../rpc/RpcDuplex");
var RpcServer_1 = require("../rpc/RpcServer");
var channel_1 = require("./channel");
var PersistentClient = (function () {
    function PersistentClient(params) {
        var _this = this;
        var _a;
        this.rpc$ = new rxjs_1.ReplaySubject(1);
        var ping = (_a = params.ping) !== null && _a !== void 0 ? _a : 15000;
        this.channel = new channel_1.PersistentChannel(params.channel);
        this.channel.open$.pipe(operators_1.filter(function (open) { return open; })).subscribe(function () {
            var close$ = _this.channel.open$.pipe(operators_1.filter(function (open) { return !open; }));
            var duplex = new RpcDuplex_1.RpcDuplex({
                client: new rpc_1.RpcClient(tslib_1.__assign(tslib_1.__assign({}, (params.client || {})), { send: function (messages) {
                        var encoded = params.codec.encoder.encode(messages);
                        _this.channel.send$(encoded).subscribe();
                    } })),
                server: new RpcServer_1.RpcServer(tslib_1.__assign(tslib_1.__assign({}, (params.server || {
                    caller: new RpcApiCaller_1.RpcApiCaller({
                        api: {},
                    }),
                    onNotification: function () { },
                })), { send: function (messages) {
                        var encoded = params.codec.encoder.encode(messages);
                        _this.channel.send$(encoded).subscribe();
                    } })),
            });
            _this.channel.message$
                .pipe(operators_1.takeUntil(close$))
                .subscribe(function (data) {
                var encoded = typeof data === 'string' ? data : new Uint8Array(data);
                var messages = params.codec.decoder.decode(encoded);
                duplex.onMessages((messages instanceof Array ? messages : [messages]), {});
            });
            if (ping) {
                rxjs_1.timer(ping, ping)
                    .pipe(operators_1.takeUntil(close$))
                    .subscribe(function () {
                    duplex.notify('.ping', undefined);
                });
            }
            if (_this.rpc)
                _this.rpc.disconnect();
            _this.rpc = duplex;
            _this.rpc$.next(duplex);
        });
    }
    PersistentClient.prototype.call$ = function (method, data) {
        return this.rpc$
            .pipe(operators_1.first(), operators_1.switchMap(function (rpc) { return rpc.call$(method, data); }));
    };
    PersistentClient.prototype.call = function (method, data) {
        return rxjs_1.firstValueFrom(this.call$(method, data));
    };
    PersistentClient.prototype.notify = function (method, data) {
        this.rpc$
            .subscribe(function (rpc) { return rpc.notify(method, data); });
    };
    PersistentClient.prototype.start = function () {
        this.channel.start();
    };
    PersistentClient.prototype.stop = function () {
        this.channel.stop();
        if (this.rpc)
            this.rpc.stop();
    };
    return PersistentClient;
}());
exports.PersistentClient = PersistentClient;
