"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RpcApiCaller = void 0;
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var constants_1 = require("./constants");
var BufferSubject_1 = require("../../../util/BufferSubject");
var error_1 = require("./error");
var RpcApiCaller = (function () {
    function RpcApiCaller(_a) {
        var api = _a.api, error = _a.error, _b = _a.preCallBufferSize, preCallBufferSize = _b === void 0 ? 10 : _b, _c = _a.maxActiveCalls, maxActiveCalls = _c === void 0 ? 50 : _c;
        this._calls = 0;
        this.api = api;
        this.error = error || new error_1.ErrorLikeErrorFormatter();
        this.preCallBufferSize = preCallBufferSize;
        this.maxActiveCalls = maxActiveCalls;
    }
    Object.defineProperty(RpcApiCaller.prototype, "activeCalls", {
        get: function () {
            return this._calls;
        },
        enumerable: false,
        configurable: true
    });
    RpcApiCaller.prototype.exists = function (name) {
        return this.api.hasOwnProperty(name);
    };
    RpcApiCaller.prototype.get = function (name) {
        if (!this.exists(name))
            throw new error_1.RpcError(constants_1.RpcServerError.NoMethodSpecified);
        return this.api[name];
    };
    RpcApiCaller.prototype.call = function (name, request, ctx) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var method, _a, error_2;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _b.trys.push([0, 10, , 11]);
                        if (this._calls >= this.maxActiveCalls)
                            throw new error_1.RpcError(constants_1.RpcServerError.TooManyActiveCalls);
                        method = this.get(name);
                        if (method.validate)
                            method.validate(request);
                        _b.label = 1;
                    case 1:
                        _b.trys.push([1, , 8, 9]);
                        this._calls++;
                        if (!method.onPreCall) return [3, 3];
                        return [4, method.onPreCall(ctx, request)];
                    case 2:
                        _b.sent();
                        _b.label = 3;
                    case 3:
                        if (!!method.isStreaming) return [3, 5];
                        return [4, method.call(ctx, request)];
                    case 4:
                        _a = _b.sent();
                        return [3, 7];
                    case 5: return [4, rxjs_1.firstValueFrom(method.call$(ctx, rxjs_1.of(request)))];
                    case 6:
                        _a = _b.sent();
                        _b.label = 7;
                    case 7: return [2, _a];
                    case 8:
                        this._calls--;
                        return [7];
                    case 9: return [3, 11];
                    case 10:
                        error_2 = _b.sent();
                        throw this.error.format(error_2);
                    case 11: return [2];
                }
            });
        });
    };
    RpcApiCaller.prototype.createCall = function (name, ctx) {
        var _this = this;
        var req$ = new rxjs_1.Subject();
        var reqUnsubscribe$ = new rxjs_1.Subject();
        try {
            if (this._calls >= this.maxActiveCalls)
                throw new error_1.RpcError(constants_1.RpcServerError.TooManyActiveCalls);
            var method = this.get(name);
            if (!method.isStreaming) {
                var response$ = rxjs_1.from((function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                    var request, response;
                    return tslib_1.__generator(this, function (_a) {
                        switch (_a.label) {
                            case 0: return [4, rxjs_1.firstValueFrom(req$.pipe(operators_1.first()))];
                            case 1:
                                request = _a.sent();
                                return [4, this.call(name, request, ctx)];
                            case 2:
                                response = _a.sent();
                                return [2, response];
                        }
                    });
                }); })());
                var res$_1 = new rxjs_1.Subject();
                response$.subscribe(res$_1);
                var $resWithErrorsFormatted_1 = res$_1.pipe(operators_1.catchError(function (error) {
                    throw _this.error.format(error);
                }));
                return { req$: req$, reqUnsubscribe$: reqUnsubscribe$, res$: $resWithErrorsFormatted_1 };
            }
            var methodStreaming_1 = method;
            var requestValidated$ = req$
                .pipe(operators_1.map(function (request) {
                if (methodStreaming_1.validate)
                    methodStreaming_1.validate(request);
                return request;
            }));
            var bufferSize = methodStreaming_1.preCallBufferSize || this.preCallBufferSize;
            var requestBuffered$_1 = new BufferSubject_1.BufferSubject(bufferSize);
            requestBuffered$_1.subscribe({ error: function () { } });
            var error$_1 = new rxjs_1.Subject();
            requestBuffered$_1.subscribe({
                error: function (error) { error$_1.error(error); },
            });
            requestValidated$.subscribe(requestBuffered$_1);
            var result$_1 = requestBuffered$_1
                .pipe(operators_1.take(1), operators_1.switchMap(function (request) {
                return methodStreaming_1.onPreCall
                    ? rxjs_1.from(methodStreaming_1.onPreCall(ctx, request))
                    : rxjs_1.from([0]);
            }), operators_1.switchMap(function () {
                Promise.resolve().then(function () {
                    requestBuffered$_1.flush();
                });
                return methodStreaming_1.call$(ctx, requestBuffered$_1)
                    .pipe(operators_1.finalize(function () {
                    error$_1.complete();
                }));
            }), operators_1.share(), operators_1.mergeWith(error$_1));
            var resultWithActiveCallTracking$_1 = rxjs_1.of(null).pipe(operators_1.tap(function () {
                _this._calls++;
            }), operators_1.switchMap(function () { return result$_1; }), operators_1.finalize(function () {
                error$_1.complete();
                _this._calls--;
            }));
            var res$ = new rxjs_1.Observable(function (observer) {
                var _a;
                var subscription = resultWithActiveCallTracking$_1.subscribe(observer);
                var timeout = (_a = methodStreaming_1.timeout) !== null && _a !== void 0 ? _a : 15000;
                var timeoutSubscription = requestBuffered$_1
                    .pipe(operators_1.catchError(function () { return rxjs_1.EMPTY; }), operators_1.mergeWith(result$_1.pipe(operators_1.catchError(function () { return rxjs_1.EMPTY; }))), operators_1.debounce(function () { return rxjs_1.interval(timeout); }))
                    .subscribe(function () {
                    var error = new error_1.RpcError(constants_1.RpcServerError.Timeout);
                    error$_1.error(error);
                });
                return function () {
                    timeoutSubscription.unsubscribe();
                    subscription.unsubscribe();
                };
            });
            var $resWithErrorsFormatted = res$.pipe(operators_1.catchError(function (error) {
                throw _this.error.format(error);
            }));
            return { req$: req$, reqUnsubscribe$: reqUnsubscribe$, res$: $resWithErrorsFormatted };
        }
        catch (error) {
            var errorFormatted = this.error.format(error);
            req$.error(errorFormatted);
            var res$ = new rxjs_1.Subject();
            res$.error(errorFormatted);
            return { req$: req$, reqUnsubscribe$: reqUnsubscribe$, res$: res$ };
        }
    };
    RpcApiCaller.prototype.call$ = function (name, request$, ctx) {
        var call = this.createCall(name, ctx);
        request$.subscribe(call.req$);
        return call.res$;
    };
    return RpcApiCaller;
}());
exports.RpcApiCaller = RpcApiCaller;
