"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RpcPersistentChannel = void 0;
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var rpc_1 = require("../rpc");
var RpcDuplex_1 = require("../rpc/RpcDuplex");
var RpcServer_1 = require("../rpc/RpcServer");
var channel_1 = require("../channel/channel");
var RpcPersistentChannel = (function () {
    function RpcPersistentChannel(params) {
        var _this = this;
        this.rpc$ = new rxjs_1.ReplaySubject(1);
        this.channel = new channel_1.PersistentChannel(params.channel);
        this.channel.open$.pipe(operators_1.filter(function (open) { return open; })).subscribe(function () {
            var close$ = _this.channel.open$.pipe(operators_1.filter(function (open) { return !open; }));
            var rpc = new RpcDuplex_1.RpcDuplex({
                client: new rpc_1.RpcClient(tslib_1.__assign(tslib_1.__assign({}, params.client), { send: function (messages) {
                        var encoded = params.codec.encoder.encode(messages);
                        _this.channel.send$(encoded).subscribe();
                    } })),
                server: new RpcServer_1.RpcServer(tslib_1.__assign(tslib_1.__assign({}, params.server), { send: function (messages) {
                        var encoded = params.codec.encoder.encode(messages);
                        _this.channel.send$(encoded).subscribe();
                    } })),
            });
            _this.channel.message$
                .pipe(operators_1.takeUntil(close$))
                .subscribe(function (data) {
                var encoded = typeof data === 'string' ? data : new Uint8Array(data);
                var messages = params.codec.decoder.decode(encoded);
                rpc.onMessages(messages, {});
            });
            if (_this.rpc)
                _this.rpc.disconnect();
            _this.rpc = rpc;
            _this.rpc$.next(rpc);
        });
    }
    RpcPersistentChannel.prototype.start = function () {
        this.channel.start();
    };
    RpcPersistentChannel.prototype.stop = function () {
        var _a;
        (_a = this.rpc) === null || _a === void 0 ? void 0 : _a.stop();
        this.channel.stop();
    };
    RpcPersistentChannel.prototype.call$ = function (method, data) {
        return this.rpc$
            .pipe(operators_1.switchMap(function (rpc) { return rpc.call$(method, data); }));
    };
    RpcPersistentChannel.prototype.notify = function (method, data) {
        this.rpc$
            .subscribe(function (rpc) { return rpc.notify(method, data); });
    };
    return RpcPersistentChannel;
}());
exports.RpcPersistentChannel = RpcPersistentChannel;
