import { ReactiveRpcRequestMessage, ReactiveRpcResponseMessage, NotificationMessage, RequestCompleteMessage, RequestDataMessage, RequestErrorMessage, ResponseUnsubscribeMessage } from '../messages/nominal';
import { RpcApi } from './types';
import { RpcServerError } from './constants';
import { ErrorFormatter } from './error';
import { RpcApiCaller } from './RpcApiCaller';
export { RpcServerError };
export interface RpcServerParams<Ctx = unknown, T = unknown> {
    caller: RpcApiCaller<any, Ctx>;
    error?: ErrorFormatter<T>;
    send: (messages: (ReactiveRpcResponseMessage<T> | NotificationMessage<T>)[]) => void;
    onNotification: (name: string, data: T | undefined, ctx: Ctx) => void;
    bufferSize?: number;
    bufferTime?: number;
}
export interface RpcServerFromApiParams<Ctx = unknown, T = unknown> extends Omit<RpcServerParams<Ctx, T>, 'onCall'> {
    api: RpcApi<Ctx, T>;
}
export declare class RpcServer<Ctx = unknown, T = unknown> {
    private readonly caller;
    readonly error: ErrorFormatter<T>;
    private readonly activeStreamCalls;
    private send;
    onSend: (messages: (ReactiveRpcResponseMessage<T> | NotificationMessage<T>)[]) => void;
    onNotification: RpcServerParams<Ctx, T>['onNotification'];
    constructor({ caller, error, send, onNotification: notify, bufferSize, bufferTime, }: RpcServerParams<Ctx, T>);
    onMessage(message: ReactiveRpcRequestMessage<T>, ctx: Ctx): void;
    onMessages(messages: ReactiveRpcRequestMessage<T>[], ctx: Ctx): void;
    stop(reason?: RpcServerError): void;
    disconnect(): void;
    private sendError;
    private execStaticCall;
    private createStreamCall;
    onRequestDataMessage(message: RequestDataMessage<T>, ctx: Ctx): void;
    onRequestCompleteMessage(message: RequestCompleteMessage<T>, ctx: Ctx): void;
    onRequestErrorMessage(message: RequestErrorMessage, ctx: Ctx): void;
    onUnsubscribeMessage(message: ResponseUnsubscribeMessage): void;
    onNotificationMessage(message: NotificationMessage<T>, ctx: Ctx): void;
}
