"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableWsBinaryReactiveRpcApi = void 0;
var binary_msgpack_1 = require("../../../common/codec/binary-msgpack");
var context_1 = require("../context");
var enableWsBinaryReactiveRpcApi = function (params) {
    var encoder = new binary_msgpack_1.Encoder();
    var decoder = new binary_msgpack_1.Decoder();
    var _a = params.route, route = _a === void 0 ? '/rpc/binary' : _a, uws = params.uws, createRpcServer = params.createRpcServer, onNotification = params.onNotification, _b = params.createContext, createContext = _b === void 0 ? context_1.createConnectionContext : _b, compression = params.compression, _c = params.idleTimeout, idleTimeout = _c === void 0 ? 0 : _c, _d = params.maxBackpressure, maxBackpressure = _d === void 0 ? 3145728 : _d, _e = params.maxPayloadLength, maxPayloadLength = _e === void 0 ? 1048576 : _e;
    uws.ws(route, {
        idleTimeout: idleTimeout,
        compression: compression,
        maxBackpressure: maxBackpressure,
        maxPayloadLength: maxPayloadLength,
        upgrade: function (res, req, context) {
            var secWebSocketKey = req.getHeader('sec-websocket-key');
            var secWebSocketProtocol = req.getHeader('sec-websocket-protocol');
            var secWebSocketExtensions = req.getHeader('sec-websocket-extensions');
            var ctx = createContext(req, res);
            res.upgrade({ ctx: ctx }, secWebSocketKey, secWebSocketProtocol, secWebSocketExtensions, context);
        },
        open: function (ws) {
            var rpc = createRpcServer({
                send: function (messages) {
                    if (ws.getBufferedAmount() > maxBackpressure)
                        return;
                    var uint8 = encoder.encode(messages);
                    ws.send(uint8, true);
                },
            });
            if (onNotification) {
                rpc.onNotification = function (name, data, ctx) {
                    onNotification(ws, name, data, ctx);
                };
            }
            ws.rpc = rpc;
        },
        message: function (ws, buf, isBinary) {
            var _a = ws, ctx = _a.ctx, rpc = _a.rpc;
            var uint8 = new Uint8Array(buf);
            var messages = decoder.decode(uint8);
            var length = messages.length;
            for (var i = 0; i < length; i++) {
                var message = messages[i];
                rpc.onMessage(message, ctx);
            }
        },
        close: function (ws, code, message) {
            ws.rpc.stop();
        },
    });
};
exports.enableWsBinaryReactiveRpcApi = enableWsBinaryReactiveRpcApi;
