"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableWsCompactReactiveRpcApi = void 0;
var compact_json_1 = require("../../../common/codec/compact-json");
var compact_msgpack_1 = require("../../../common/codec/compact-msgpack");
var NotificationMessage_1 = require("../../../common/messages/nominal/NotificationMessage");
var context_1 = require("../context");
var enableWsCompactReactiveRpcApi = function (params) {
    var encoderJson = new compact_json_1.Encoder();
    var decoderJson = new compact_json_1.Decoder();
    var encoderMsgPack = new compact_msgpack_1.Encoder();
    var decoderMsgPack = new compact_msgpack_1.Decoder();
    var invalidPayloadNotification = new NotificationMessage_1.NotificationMessage('.err', 'CODING');
    var invalidPayloadJson = encoderJson.encode([invalidPayloadNotification]);
    var invalidPayloadMsgPack = encoderMsgPack.encode([invalidPayloadNotification]);
    var _a = params.route, route = _a === void 0 ? '/rpc/compact' : _a, uws = params.uws, createRpcServer = params.createRpcServer, onNotification = params.onNotification, _b = params.createContext, createContext = _b === void 0 ? context_1.createConnectionContext : _b, compression = params.compression, _c = params.idleTimeout, idleTimeout = _c === void 0 ? 0 : _c, _d = params.maxBackpressure, maxBackpressure = _d === void 0 ? 3145728 : _d, _e = params.maxPayloadLength, maxPayloadLength = _e === void 0 ? 1048576 : _e;
    uws.ws(route, {
        idleTimeout: idleTimeout,
        compression: compression,
        maxBackpressure: maxBackpressure,
        maxPayloadLength: maxPayloadLength,
        upgrade: function (res, req, context) {
            var secWebSocketKey = req.getHeader('sec-websocket-key');
            var secWebSocketProtocol = req.getHeader('sec-websocket-protocol');
            var secWebSocketExtensions = req.getHeader('sec-websocket-extensions');
            var ctx = createContext(req, res);
            var isBinary = secWebSocketProtocol.indexOf('MsgPack') > -1;
            res.upgrade({ ctx: ctx, isBinary: isBinary }, secWebSocketKey, secWebSocketProtocol, secWebSocketExtensions, context);
        },
        open: function (ws) {
            var rpc = createRpcServer({
                send: function (messages) {
                    if (ws.getBufferedAmount() > maxBackpressure)
                        return;
                    var isBinary = ws.isBinary;
                    var encoded = isBinary ? encoderMsgPack.encode(messages) : encoderJson.encode(messages);
                    ws.send(encoded, isBinary);
                },
            });
            if (onNotification) {
                rpc.onNotification = function (name, data, ctx) {
                    onNotification(ws, name, data, ctx);
                };
            }
            ws.rpc = rpc;
        },
        message: function (ws, buf, isBinary) {
            var _a = ws, ctx = _a.ctx, rpc = _a.rpc;
            var messages;
            try {
                messages = isBinary
                    ? decoderMsgPack.decode(new Uint8Array(buf))
                    : decoderJson.decode(Buffer.from(buf).toString('utf8'));
            }
            catch (error) {
                if (ws.getBufferedAmount() > maxBackpressure)
                    return;
                ws.send(isBinary ? invalidPayloadMsgPack : invalidPayloadJson, isBinary);
                return;
            }
            if (messages instanceof Array)
                rpc.onMessages(messages, ctx);
            else
                rpc.onMessage(messages, ctx);
        },
        close: function (ws, code, message) {
            ws.rpc.stop();
        },
    });
};
exports.enableWsCompactReactiveRpcApi = enableWsCompactReactiveRpcApi;
